# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import numpy as np

from nibabel import load, Nifti1Image

from . import discrete_domain as ddom


##############################################################################
# class SubDomains
##############################################################################
class SubDomains(object):
    """
    This is a class to represent multiple ROI objects, where the
    reference to a given domain is explicit.

    A mutliple ROI object is a set of ROI defined on a given domain,
    each having its own 'region-level' characteristics (ROI features).

    Every voxel of the domain can have its own characteristics yet,
    defined at the 'voxel-level', but those features can only be accessed
    familywise (i.e. the values are grouped by ROI).

    Parameters
    ----------
    k : int
      Number of ROI in the SubDomains object
    label : array of shape (domain.size), dtype=np.int
      An array use to define which voxel belongs to which ROI.
      The label values greater than -1 correspond to subregions
      labelling. The labels are recomputed so as to be consecutive
      integers.
      The labels should not be accessed outside this class. One has to
      use the API mapping methods instead.
    features : dict {str: list of object, length=self.k}
      Describe the voxels features, grouped by ROI
    roi_features : dict {str: array-like, shape=(self.k, roi_feature_dim)
      Describe the ROI features. A special feature, `id`, is read-only and
      is used to give an unique identifier for region, which is persistent
      through the MROI objects manipulations. On should access the different
      ROI's features using ids.

    """

    def __init__(self, domain, label, id=None):
        """Initialize subdomains instance

        Parameters
        ----------
        domain: ROI instance
          defines the spatial context of the SubDomains
        label: array of shape (domain.size), dtype=np.int,
          An array use to define which voxel belongs to which ROI.
          The label values greater than -1 correspond to subregions
          labelling. The labels are recomputed so as to be consecutive
          integers.
          The labels should not be accessed outside this class. One has to
          use the select_id() mapping method instead.
        id: array of shape (n_roi)
          Define the ROI identifiers. Once an id has been associated to a ROI
          it becomes impossible to change it using the API. Hence, one
          should access ROI through their id to avoid hazardous manipulations.

        """
        # check that label size is consistent with domain
        if np.size(label) != domain.size:
            raise ValueError('inconsistent labels and domains specification')
        self.domain = domain
        self.label = np.reshape(label, label.size).astype(np.int)
        # use continuous labels
        self.recompute_labels()

        # initialize empty feature/roi_feature dictionaries
        self.features = {}
        self.roi_features = {}
        # set `id` feature: unique and persistent identifier for each roi
        if id is None:
            # ids correspond to initial labels
            self.set_roi_feature('id', np.arange(self.k))
        else:
            # use user-provided ids
            if len(id) != self.k:
                raise ValueError("incorrect shape for `id`")
            else:
                self.set_roi_feature('id', id)

    ###
    # Methods for internal use: id and labels manipulations
    ###
    def recompute_labels(self):
        """Redefine labels so that they are consecutive integers.

        Labels are used as a map to associate voxels to a given ROI.
        It is an inner object that should not be accessed outside this class.
        The number of nodes is updated appropriately.

        Notes
        -----
        This method must be called everytime the MROI structure is modified.
        """
        lmap = np.unique(self.label[self.label > - 1])
        for i, k in enumerate(lmap):
            self.label[self.label == k] = i
        # number of ROIs: number of labels > -1
        self.k = np.amax(self.label) + 1

    def get_id(self):
        """Return ROI's id list.

        Users must access ROIs with the use of the identifiers of this list
        and the methods that give access to their properties/features.

        """
        return self.get_roi_feature('id')

    def select_id(self, id, roi=True):
        """Convert a ROI id into an index to be used to index features safely.

        Parameters
        ----------
        id : any hashable type, must be in self.get_id()
          The id of the region one wants to access.
        roi : bool
          If True (default), return the ROI index in the ROI list.
          If False, return the indices of the voxels of the ROI with the given
          id. That way, internal access to self.label can be made.

        Returns
        -------
        index : int or np.array of shape (roi.size, )
          Either the position of the ROI in the ROI list (if roi == True),
          or the positions of the voxels of the ROI with id `id`
          with respect to the self.label array.

        """
        if id not in self.get_id():
            raise ValueError("Unexisting `id` provided")
        if roi:
            index = int(np.where(self.get_id() == id)[0])
        else:
            index = np.where(self.label == np.where(self.get_id() == id)[0])[0]
        return index

    ###
    # General purpose methods
    ###
    def copy(self):
        """Returns a copy of self.

        Note that self.domain is not copied.

        """
        cp = SubDomains(self.domain, self.label.copy(), id=self.get_id())
        for fid in self.features.keys():
            f = self.get_feature(fid)
            sf = [np.array(f[k]).copy() for k in range(self.k)]
            cp.set_feature(fid, sf)
        for fid in self.roi_features.keys():
            cp.set_roi_feature(fid, self.get_roi_feature(fid).copy())
        return cp

    ###
    # Getters for very basic features or roi features
    ###
    def get_coord(self, id=None):
        """Get coordinates of ROI's voxels

        Parameters
        ----------
        id: any hashable type
          Id of the ROI from which we want the voxels' coordinates.
          Can be None (default) if we want all ROIs's voxels coordinates.

        Returns
        -------
        coords: array-like, shape=(roi_size, domain_dimension)
          if an id is provided,
             or list of arrays of shape(roi_size, domain_dimension)
          if no id provided (default)

        """
        if id is not None:
            coords = self.domain.coord[self.select_id(id, roi=False)]
        else:
            coords = [self.domain.coord[self.select_id(k, roi=False)]
                      for k in self.get_id()]
        return coords

    def get_size(self, id=None):
        """Get ROI size (counted in terms of voxels)

        Parameters
        ----------
        id: any hashable type
          Id of the ROI from which we want to get the size.
          Can be None (default) if we want all ROIs's sizes.

        Returns
        -------
        size: int
          if an id is provided,
             or list of int
          if no id provided (default)

        """
        if id is not None:
            size = np.size(self.select_id(id, roi=False))
        else:
            size = np.array(
                [np.size(self.select_id(k, roi=False)) for k in self.get_id()])
        return size

    def get_local_volume(self, id=None):
        """Get volume of ROI's voxels

        Parameters
        ----------
        id: any hashable type
          Id of the ROI from which we want the voxels' volumes.
          Can be None (default) if we want all ROIs's voxels volumes.

        Returns
        -------
        loc_volume: array-like, shape=(roi_size, ),
          if an id is provided,
             or list of arrays of shape(roi_size, )
          if no id provided (default)

        """
        if id is not None:
            loc_volume = self.domain.local_volume[
                self.select_id(id, roi=False)]
        else:
            loc_volume = [self.domain.local_volume[
                    self.select_id(k, roi=False)] for k in self.get_id()]
        return loc_volume

    def get_volume(self, id=None):
        """Get ROI volume

        Parameters
        ----------
        id: any hashable type
          Id of the ROI from which we want to get the volume.
          Can be None (default) if we want all ROIs's volumes.

        Returns
        -------
        volume : float
          if an id is provided,
             or list of float
          if no id provided (default)

        """
        if id is not None:
            volume = np.sum(self.get_local_volume(id))
        else:
            volume = np.asarray([np.sum(k) for k in self.get_local_volume()])
        return volume

    ###
    # Methods for features manipulation (user level)
    ###
    def get_feature(self, fid, id=None):
        """Return a voxel-wise feature, grouped by ROI.

        Parameters
        ----------
        fid: str,
          Feature to be returned
        id: any hashable type
          Id of the ROI from which we want to get the feature.
          Can be None (default) if we want all ROIs's features.

        Returns
        -------
        feature: array-like, shape=(roi_size, feature_dim)
          if an id is provided,
             or list of arrays, shape=(roi_size, feature_dim)
          if no id provided (default)

        """
        if fid not in self.features:
            raise ValueError("the `%s` feature does not exist" % fid)
        if id is not None:
            feature = np.asarray(self.features[fid][self.select_id(id)])
        else:
            feature = self.features[fid]
        return feature

    def set_feature(self, fid, data, id=None, override=False):
        """Append or modify a feature

        Parameters
        ----------
        fid : str
          feature identifier
        data: list or array
          The feature data. Can be a list of self.k arrays of
          shape(self.size[k], p) or array of shape(self.size[k])
        id: any hashable type, optional
          Id of the ROI from which we want to set the feature.
          Can be None (default) if we want to set all ROIs's features.
        override: bool, optional
          Allow feature overriding

        Note that we cannot create a feature having the same name than
        a ROI feature.

        """
        # ensure that the `id` field will not be modified
        if fid == 'id':
            override = False
        # check the feature is already present if setting a single roi
        if fid not in self.features and len(data) != self.k:
            raise ValueError("`%s` feature does not exist, create it first"
                             % fid)
        if fid in self.roi_features:
            raise ValueError("a roi_feature called `%s` already exists" % fid)
        # check we will not override anything
        if fid in self.features and not override:
            #TODO: raise a warning
            return
        # modify one particular region
        if id is not None:
            # check data size
            roi_size = self.get_size(id)
            if len(data) != roi_size:
                raise ValueError("data for region `%i` should have length %i"
                                 % (id, roi_size))
            # update feature
            the_feature = self.get_feature(fid, id)
            the_feature[self.select_id(id)] = data
        # modify all regions
        else:
            # check data size
            if len(data) != self.k:
                raise ValueError("data should have length %i" % self.k)
            for k in self.get_id():
                if len(data[self.select_id(k)]) != self.get_size(k):
                    raise ValueError('Wrong data size for region `%i`' % k)
            self.features.update({fid: data})

    def representative_feature(self, fid, method='mean', id=None,
                               assess_quality=False):
        """Compute a ROI representative of a given feature.

        Parameters
        ----------
        fid : str
          Feature id
        method : str, optional
          Method used to compute a representative.
          Chosen among 'mean' (default), 'max', 'median', 'min',
          'weighted mean'.
        id : any hashable type, optional
          Id of the ROI from which we want to extract a representative feature.
          Can be None (default) if we want to get all ROIs's representatives.
        assess_quality: bool, optional
          If True, a new roi feature is created, which represent the quality of
          the feature representative (the number of non-nan value for the
          feature over the ROI size).  Default is False.

        Returns
        -------
        summary_feature: np.ndarray, shape=(self.k, feature_dim)
          Representative feature computed according to `method`.
        """
        rf = []
        eps = 1.e-15
        feature_quality = np.zeros(self.k)
        for i, k in enumerate(self.get_id()):
            f = self.get_feature(fid, k)
            # NaN-resistant representative
            if f.ndim == 2:
                nan = np.isnan(f.sum(1))
            else:
                nan = np.isnan(f)
            # feature quality
            feature_quality[i] = (~nan).sum() / float(nan.size)
            # compute representative
            if method == "mean":
                rf.append(np.mean(f[~nan], 0))
            if method == "weighted mean":
                lvk = self.get_local_volume(k)[~nan]
                tmp = np.dot(lvk, f[~nan].reshape((-1, 1))) / \
                    np.maximum(eps, np.sum(lvk))
                rf.append(tmp)
            if method == "min":
                rf.append(np.min(f[~nan]))
            if method == "max":
                rf.append(np.max(f[~nan]))
            if method == "median":
                rf.append(np.median(f[~nan], 0))
        if id is not None:
            summary_feature = rf[self.select_id(id)]
        else:
            summary_feature = rf

        if assess_quality:
            self.set_roi_feature('%s_quality' % fid, feature_quality)
        return np.array(summary_feature)

    def remove_feature(self, fid):
        """Remove a certain feature

        Parameters
        ----------
        fid: str
          Feature id

        Returns
        -------
        f : object
            The removed feature.
        """
        return self.features.pop(fid)

    def feature_to_voxel_map(self, fid, roi=False, method="mean"):
        """Convert a feature to a flat voxel-mapping array.

        Parameters
        ----------
        fid: str
          Identifier of the feature to be mapped.
        roi: bool, optional
          If True, compute the map from a ROI feature.
        method: str, optional
          Representative feature computation method if `fid` is a feature
          and `roi` is True.

        Returns
        -------
        res: array-like, shape=(domain.size, feature_dim)
          A flat array, giving the correspondence between voxels
          and the feature.
        """
        res = np.zeros(self.label.size)
        if not roi:
            f = self.get_feature(fid)
            for id in self.get_id():
                res[self.select_id(id, roi=False)] = f[self.select_id(id)]
        else:
            if fid in self.roi_features.keys():
                f = self.get_roi_feature(fid)
                for id in self.get_id():
                    res[self.select_id(id, roi=False)] = f[self.select_id(id)]
            elif fid in self.features.keys():
                f = self.representative_feature(fid, method=method)
                for id in self.get_id():
                    res[self.select_id(id, roi=False)] = f[self.select_id(id)]
            else:
                raise ValueError("Wrong feature id provided")
        return res

    def integrate(self, fid=None, id=None):
        """Integrate  certain feature on each ROI and return the k results

        Parameters
        ----------
        fid : str
          Feature identifier.
          By default, the 1 function is integrated, yielding ROI volumes.
        id: any hashable type
          The ROI on which we want to integrate.
          Can be None if we want the results for every region.

        Returns
        -------
        lsum = array of shape (self.k, self.feature[fid].shape[1]),
          The results

        """
        if fid == None:
            # integrate the 1 function if no feature id provided
            if id is not None:
                lsum = self.get_volume(id)
            else:
                lsum = [self.get_volume(k) for k in self.get_id()]
        else:
            if id is not None:
                slvk = np.expand_dims(self.get_local_volume(id), 1)
                sfk = self.get_feature(fid, id)
                sfk = np.reshape(sfk, (-1, 1))
                lsum = np.sum(sfk * slvk, 0)
            else:
                lsum = []
                for k in self.get_id():
                    slvk = np.expand_dims(self.get_local_volume(k), 1)
                    sfk = self.get_feature(fid, k)
                    sfk = np.reshape(sfk, (-1, 1))
                    sumk = np.sum(sfk * slvk, 0)
                    lsum.append(sumk)
        return np.array(lsum)

    def plot_feature(self, fid, ax=None):
        """Boxplot the distribution of features within ROIs.
        Note that this assumes 1-d features.

        Parameters
        ----------
        fid: string
             the feature identifier
        ax: axis handle, optional

        """
        f = self.get_feature(fid)
        if ax is None:
            import matplotlib.pylab as mp
            mp.figure()
            ax = mp.subplot(111)
        ax.boxplot(f)
        ax.set_title('ROI-level distribution for feature %s' % fid)
        ax.set_xlabel('Region index')
        ax.set_xticks(np.arange(1, self.k + 1))
        return ax

    ###
    # Methods for ROI features manipulation (user level)
    ###
    def get_roi_feature(self, fid, id=None):
        """
        """
        if id is not None:
            feature = self.roi_features[fid][self.select_id(id)]
        else:
            feature = np.asarray(self.roi_features[fid])
        return feature

    def set_roi_feature(self, fid, data, id=None, override=False):
        """Append or modify a ROI feature

        Parameters
        ----------
        fid: str,
          feature identifier
        data: list of self.k features or a single feature
          The ROI feature data
        id: any hashable type
          Id of the ROI of which we want to set the ROI feature.
          Can be None (default) if we want to set all ROIs's ROI features.
        override: bool, optional,
                  Allow feature overriding

        Note that we cannot create a ROI feature having the same name than
        a feature.
        Note that the `id` feature cannot be modified as an internal
        component.

        """
        # check we do not modify the `id` feature
        if 'id' in self.roi_features.keys() and fid == 'id':
            return
        # check we will not override anything
        if fid in self.roi_features and not override:
            #TODO: raise a warning
            return
        # check the feature is already present if setting a single roi
        if fid not in self.roi_features and len(data) != self.k:
            raise ValueError("`%s` feature does not exist, create it first")
        if fid in self.features:
            raise ValueError("a feature called `%s` already exists" % fid)

        # modify one particular region
        if id is not None:
            # check data size
            if len(data) != 1:
                raise ValueError("data for region `%i` should have length 1")
            # update feature
            the_feature = self.get_roi_feature(fid)
            the_feature[self.select_id(id)] = data
        else:
            # check data size
            if len(data) != self.k:
                raise ValueError("data should have length %i" % self.k)
            self.roi_features.update({fid: data})

    def remove_roi_feature(self, fid):
        """Remove a certain ROI feature.

        The `id` ROI feature cannot be removed.

        Returns
        -------
        f : object
            The removed Roi feature.
        """
        if fid != 'id':
            feature = self.roi_features.pop(fid)
        else:
            feature = self.get_id()
        return feature
        #TODO: raise a warning otherwise

    def to_image(self, fid=None, roi=False, method="mean", descrip=None):
        """Generates a label image that represents self.

        Parameters
        ----------
        fid: str,
          Feature to be represented. If None, a binary image of the MROI
          domain will be we created.
        roi: bool,
          Whether or not to write the desired feature as a ROI one.
          (i.e. a ROI feature corresponding to `fid` will be looked upon,
          and if not found, a representative feature will be computed from
          the `fid` feature).
        method: str,
          If a feature is written as a ROI feature, this keyword tweaks
          the way the representative feature is computed.
        descrip: str,
          Description of the image, to be written in its header.

        Notes
        -----
        Requires that self.dom is an ddom.NDGridDomain

        Returns
        -------
        nim : nibabel nifti image
          Nifti image corresponding to the ROI feature to be written.

        """
        if not isinstance(self.domain, ddom.NDGridDomain):
            print 'self.domain is not an NDGridDomain; nothing was written.'
            return None

        if fid is None:
            # write a binary representation of the domain if no fid provided
            nim = self.domain.to_image(data=(self.label != -1).astype(np.int32))
            if descrip is None:
                descrip = 'binary representation of MROI'
        else:
            data = -np.ones(self.label.size, dtype=np.int32)
            tmp_image = self.domain.to_image()
            mask = tmp_image.get_data().copy().astype(bool)
            if not roi:
                # write a feature
                if fid not in self.features:
                    raise ValueError("`%s` feature could not be found" % fid)
                for i in self.get_id():
                    data[self.select_id(i, roi=False)] = \
                        self.get_feature(fid, i)
            else:
                # write a roi feature
                if fid in self.roi_features:
                    # write from existing roi feature
                    for i in self.get_id():
                        data[self.select_id(i, roi=False)] = \
                            self.get_roi_feature(
                            fid, i)
                elif fid in self.features:
                    # write from representative feature
                    summary_feature = self.representative_feature(
                        fid, method=method)
                    for i in self.get_id():
                        data[self.select_id(i, roi=False)] = \
                            summary_feature[self.select_id(i)]
            # MROI object was defined on a masked image: we square it back.
            wdata = -np.ones(mask.shape, data.dtype)
            wdata[mask] = data
            nim = Nifti1Image(wdata, tmp_image.get_affine())
        # set description of the image
        if descrip is not None:
            nim.get_header()['descrip'] = descrip
        return nim

    ###
    # ROIs structure manipulation
    ###
    def select_roi(self, id_list):
        """Returns an instance of MROI with only the subset of chosen ROIs.

        Parameters
        ----------
        id_list: list of id (any hashable type)
          The id of the ROI to be kept in the structure.

        """
        # handle the case of an empty selection
        if len(id_list) == 0:
            self = SubDomains(self.domain, -np.ones(self.label.size))
            return
        # convert id to indices
        id_list_pos = np.ravel([self.select_id(k) for k in id_list])
        # set new labels (= map between voxels and ROI)
        for id in self.get_id():
            if id not in id_list:
                self.label[self.select_id(id, roi=False)] = -1
        self.recompute_labels()
        self.roi_features['id'] = np.ravel([id_list])

        # set new features
        # (it's ok to do that after labels and id modification since we are
        # poping out the former features and use the former id indices)
        for fid in self.features.keys():
            f = self.remove_feature(fid)
            sf = [f[id] for id in id_list_pos]
            self.set_feature(fid, sf)
        # set new ROI features
        # (it's ok to do that after labels and id modification since we are
        # poping out the former features and use the former id indices)
        for fid in self.roi_features.keys():
            if fid != 'id':
                f = self.remove_roi_feature(fid)
                sf = np.ravel(f[id_list_pos])
                self.set_roi_feature(fid, sf)


def subdomain_from_array(labels, affine=None, nn=0):
    """Return a SubDomain from an n-d int array

    Parameters
    ----------
    label: np.array instance
      A supposedly boolean array that yields the regions.
    affine: np.array, optional
      Affine transform that maps the array coordinates
      to some embedding space by default, this is np.eye(dim+1, dim+1).
    nn: int,
      Neighboring system considered.
      Unused at the moment.

    Notes
    -----
    Only labels > -1 are considered.

    """
    dom = ddom.grid_domain_from_binary_array(
        np.ones(labels.shape), affine=affine, nn=nn)
    return SubDomains(dom, labels.astype(np.int))


def subdomain_from_image(mim, nn=18):
    """Return a SubDomain instance from the input mask image.

    Parameters
    ----------
    mim: NiftiIImage instance, or string path toward such an image
      supposedly a label image
    nn: int, optional
      Neighboring system considered from the image can be 6, 18 or 26.

    Returns
    -------
    The MultipleROI instance

    Notes
    -----
    Only labels > -1 are considered

    """
    if isinstance(mim, basestring):
        iim = load(mim)
    else:
        iim = mim

    return subdomain_from_array(iim.get_data(), iim.get_affine(), nn)


def subdomain_from_position_and_image(nim, pos):
    """Keep the set of labels of the image corresponding to a certain index
    so that their position is closest to the prescribed one.

    Parameters
    ----------
    mim: NiftiIImage instance, or string path toward such an image
         supposedly a label image
    pos: array of shape(3) or list of length 3,
         the prescribed position

    """
    tmp = subdomain_from_image(nim)
    coord = np.array([tmp.domain.coord[tmp.label == k].mean(0)
                      for k in range(tmp.k)])
    idx = ((coord - pos) ** 2).sum(1).argmin()
    return subdomain_from_array(nim.get_data() == idx, nim.get_affine())


def subdomain_from_balls(domain, positions, radii):
    """Create discrete ROIs as a set of balls within a certain
    coordinate systems.

    Parameters
    ----------
    domain: StructuredDomain instance,
        the description of a discrete domain
    positions: array of shape(k, dim):
        the positions of the balls
    radii: array of shape(k):
        the sphere radii

    """
    # checks
    if np.size(positions) == positions.shape[0]:
        positions = np.reshape(positions, (positions.size), 1)
    if positions.shape[1] != domain.em_dim:
        raise ValueError('incompatible dimensions for domain and positions')
    if positions.shape[0] != np.size(radii):
        raise ValueError('incompatible positions and radii provided')

    label = - np.ones(domain.size)

    for k in range(radii.size):
        supp = np.sum((domain.coord - positions[k]) ** 2, 1) < radii[k] ** 2
        label[supp] = k

    return SubDomains(domain, label)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This figure is meant to represent an event-type design with
events at times [0,4,8,12,16] and amplitudes [0,1.1,2.3,0.9,0.3].
"""

import pylab
import numpy as np

pylab.scatter([0,4,8,12,16], [0,1.1,2.3,0.9,0.3], c='r', marker='o')

a = pylab.gca()
a.set_yticks([0,2])
a.set_xlabel('Time')
a.set_ylabel('Amplitude')




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os


def configuration(parent_package='', top_path=None):

    from numpy.distutils.misc_util import Configuration

    config = Configuration('registration', parent_package, top_path)
    config.add_subpackage('tests')
    config.add_include_dirs(config.name.replace('.', os.sep))
    config.add_extension(
        '_registration',
        sources=['_registration.pyx',
                 'joint_histogram.c',
                 'wichmann_prng.c',
                 'cubic_spline.c',
                 'polyaffine.c'])
    return config


if __name__ == '__main__':
    print('This is the wrong setup.py file to run')



"""
Test the Von-Mises-Fisher mixture model

Author : Bertrand Thirion, 2010
"""

import numpy as np

from ..von_mises_fisher_mixture import (VonMisesMixture,
                                        sphere_density,
                                        select_vmm,
                                        select_vmm_cv)


from nose.tools import assert_true, assert_equal


def test_spherical_area():
    # test the co_labelling functionality
    points, area = sphere_density(100)
    assert_true(np.abs(area.sum()-4*np.pi)<1.e-2)


def test_von_mises_fisher_density():
    # test that a density is indeed computed on the unit sphere for a
    # one-component and three-component model (k == 1, 3)
    x = np.random.randn(100, 3)
    x = (x.T/np.sqrt(np.sum(x**2, 1))).T
    s, area = sphere_density(100)
    for k in (1, 3):
        for precision in [.1, 1., 10., 100.]:
            for null_class in (False, True):
                vmd = VonMisesMixture(k, precision, null_class=null_class)
                vmd.estimate(x)
                # check that it sums to 1
                assert_true(np.abs((vmd.mixture_density(s)*area).sum() - 1)
                            < 1e-2)


def test_dimension_selection_bic():
    # Tests whether dimension selection yields correct results
    x1 = [0.6, 0.48, 0.64]
    x2 = [-0.8, 0.48, 0.36]
    x3 = [0.48, 0.64, -0.6]
    x = np.random.randn(200, 3) * .1
    x[:40] += x1
    x[40:150] += x2
    x[150:] += x3
    x = (x.T / np.sqrt(np.sum(x**2, 1))).T

    precision = 100.
    my_vmm = select_vmm(range(1,8), precision, False, x)
    assert_equal(my_vmm.k, 3)


def test_dimension_selection_cv():
    # Tests the dimension selection using cross validation
    x1 = [1, 0, 0]
    x2 = [-1, 0, 0]
    x = np.random.randn(20, 3)*.1
    x[0::2] += x1
    x[1::2] += x2
    x = (x.T / np.sqrt(np.sum(x**2,1))).T

    precision = 50.
    sub = np.repeat(np.arange(10), 2)
    my_vmm = select_vmm_cv(range(1,8), precision, x, cv_index=sub,
                           null_class=False, ninit=5)
    z = np.argmax(my_vmm.responsibilities(x), 1)
    assert_true(len(np.unique(z))>1)
    assert_true(len(np.unique(z))<4)


if __name__ == '__main__':
    import nose
    nose.run(argv=['', __file__])



#!/usr/bin/env python


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration
    # We need this because libcstat.a is linked to lapack, which can
    # be a fortran library, and the linker needs this information.
    from numpy.distutils.system_info import get_info
    lapack_info = get_info('lapack_opt', 0)
    if 'libraries' not in lapack_info:
        # But on OSX that may not give us what we need, so try with 'lapack'
        # instead.  NOTE: scipy.linalg uses lapack_opt, not 'lapack'...
        lapack_info = get_info('lapack', 0)

    config = Configuration('utils', parent_package, top_path)
    config.add_subpackage('tests')
    config.add_extension(
                'routines',
                sources=['routines.pyx'],
                libraries=['cstat'],
                extra_info=lapack_info,
                )

    return config


if __name__ == '__main__':
    print('This is the wrong setup.py file to run')





def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('spatial_models', parent_package, top_path)
    config.add_subpackage('tests')
    return config


if __name__ == '__main__':
    print('This is the wrong setup.py file to run')



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Tests for the temporary matlab file module."""

# Stdlib imports
import os
import tempfile

# Our own imports
from nipy.interfaces.matlab import mlab_tempfile

# Functions, classes and other top-level code
def check_mlab_tempfile(dir):
    """Helper function for testing the mlab temp file creation."""

    try:
        f = mlab_tempfile(dir)
    except OSError,msg:
        if not os.path.isdir(dir) and 'No such file or directory' in msg:
            # This is OK, it's the expected error
            return True
        else:
            raise
    else:
        f.close()


def test_mlab_tempfile():
    for dir in [None,tempfile.tempdir,tempfile.mkdtemp()]:
        yield check_mlab_tempfile,dir



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import with_statement

import numpy as np

from nibabel.affines import from_matvec

from nipy.io.api import load_image, save_image
from nipy.core import api
from nipy.core.reference.coordinate_map import (
    AffineTransform as AT)
from nipy.core.reference.coordinate_system import (
    CoordinateSystem as CS)
from nipy.core.reference.spaces import mni_csm

from nibabel.tmpdirs import InTemporaryDirectory

from nose.tools import (assert_true, assert_false, assert_equal,
                        assert_not_equal)

from numpy.testing import assert_array_almost_equal

from nipy.testing import funcfile

TMP_FNAME = 'afile.nii'


def test_save1():
    # A test to ensure that when a file is saved, the affine and the
    # data agree. This image comes from a NIFTI file
    img = load_image(funcfile)
    with InTemporaryDirectory():
        save_image(img, TMP_FNAME)
        img2 = load_image(TMP_FNAME)
        assert_array_almost_equal(img.affine, img2.affine)
        assert_equal(img.shape, img2.shape)
        assert_array_almost_equal(img2.get_data(), img.get_data())
        del img2


def test_save2():
    # A test to ensure that when a file is saved, the affine and the
    # data agree. This image comes from a NIFTI file 
    shape = (13,5,7,3)
    step = np.array([3.45,2.3,4.5,6.93])
    cmap = api.AffineTransform.from_start_step('ijkt', 'xyzt', [1,3,5,0], step)
    data = np.random.standard_normal(shape)
    img = api.Image(data, cmap)
    with InTemporaryDirectory():
        save_image(img, TMP_FNAME)
        img2 = load_image(TMP_FNAME)
        assert_array_almost_equal(img.affine, img2.affine)
        assert_equal(img.shape, img2.shape)
        assert_array_almost_equal(img2.get_data(), img.get_data())
        del img2


def test_save2b():
    # A test to ensure that when a file is saved, the affine and the
    # data agree. This image comes from a NIFTI file.  This example has a
    # non-diagonal affine matrix for the spatial part, but is 'diagonal' for the
    # space part.
    #
    # make a 5x5 transformation (for 4d image)
    step = np.array([3.45, 2.3, 4.5, 6.9])
    A = np.random.standard_normal((3,3))
    B = np.diag(list(step)+[1])
    B[:3, :3] = A
    shape = (13,5,7,3)
    cmap = api.vox2mni(B)
    data = np.random.standard_normal(shape)
    img = api.Image(data, cmap)
    with InTemporaryDirectory():
        save_image(img, TMP_FNAME)
        img2 = load_image(TMP_FNAME)
        assert_array_almost_equal(img.affine, img2.affine)
        assert_equal(img.shape, img2.shape)
        assert_array_almost_equal(img2.get_data(), img.get_data())
        del img2


def test_save3():
    # A test to ensure that when a file is saved, the affine
    # and the data agree. In this case, things don't agree:
    # i) the pixdim is off
    # ii) makes the affine off
    step = np.array([3.45,2.3,4.5,6.9])
    shape = (13,5,7,3)
    mni_xyz = mni_csm(3).coord_names
    cmap = AT(CS('jkli'),
              CS(('t',) + mni_xyz[::-1]),
              from_matvec(np.diag([0,3,5,1]), step))
    data = np.random.standard_normal(shape)
    img = api.Image(data, cmap)
    # with InTemporaryDirectory():
    with InTemporaryDirectory():
        save_image(img, TMP_FNAME)
        tmp = load_image(TMP_FNAME)
        # Detach image from file so we can delete it
        data = tmp.get_data().copy()
        img2 = api.Image(data, tmp.coordmap, tmp.metadata)
        del tmp
    assert_equal(tuple([img.shape[l] for l in [3,2,1,0]]), img2.shape)
    a = np.transpose(img.get_data(), [3,2,1,0])
    assert_false(np.allclose(img.affine, img2.affine))
    assert_true(np.allclose(a, img2.get_data()))


def test_save4():
    # Same as test_save3 except we have reordered the 'ijk' input axes.
    shape = (13,5,7,3)
    step = np.array([3.45,2.3,4.5,6.9])
    # When the input coords are in the 'ljki' order, the affines get
    # rearranged.  Note that the 'start' below, must be 0 for
    # non-spatial dimensions, because we have no way to store them in
    # most cases.  For example, a 'start' of [1,5,3,1] would be lost on
    # reload
    mni_xyz = mni_csm(3).coord_names
    cmap = AT(CS('tkji'),
              CS((('t',) + mni_xyz[::-1])),
              from_matvec(np.diag([2., 3, 5, 1]), step))
    data = np.random.standard_normal(shape)
    img = api.Image(data, cmap)
    with InTemporaryDirectory():
        save_image(img, TMP_FNAME)
        tmp = load_image(TMP_FNAME)
        data = tmp.get_data().copy()
        # Detach image from file so we can delete it
        img2 = api.Image(data, tmp.coordmap, tmp.metadata)
        del tmp
    P = np.array([[0,0,0,1,0],
                  [0,0,1,0,0],
                  [0,1,0,0,0],
                  [1,0,0,0,0],
                  [0,0,0,0,1]])
    res = np.dot(P, np.dot(img.affine, P.T))
    # the step part of the affine should be set correctly
    assert_array_almost_equal(res[:4,:4], img2.affine[:4,:4])
    # start in the spatial dimensions should be set correctly
    assert_array_almost_equal(res[:3,-1], img2.affine[:3,-1])
    # start in the time dimension should be 3.45 as in img, because NIFTI stores
    # the time offset in hdr[``toffset``]
    assert_not_equal(res[3,-1], img2.affine[3,-1])
    assert_equal(res[3,-1], 3.45)
    # shapes should be reversed because img has coordinates reversed
    assert_equal(img.shape[::-1], img2.shape)
    # data should be transposed because coordinates are reversed
    assert_array_almost_equal(
        np.transpose(img2.get_data(),[3,2,1,0]),
        img.get_data())
    # coordinate names should be reversed as well
    assert_equal(img2.coordmap.function_domain.coord_names,
                 img.coordmap.function_domain.coord_names[::-1])
    assert_equal(img2.coordmap.function_domain.coord_names,
                 ('i', 'j', 'k', 't'))



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Conversion mechansims for IO and interaction between volumetric datasets 
and other type of neuroimaging data.
"""
import os

import numpy as np

import nibabel as nib
from nibabel.spatialimages import SpatialImage

from .volumes.volume_img import VolumeImg

def as_volume_img(obj, copy=True, squeeze=True, world_space=None):
    """ Convert the input to a VolumeImg.

        Parameters
        ----------
        obj : filename, pynifti or brifti object, or volume dataset.
            Input object, in any form that can be converted to a
            VolumeImg. This includes Nifti filenames, pynifti or brifti
            objects, or other volumetric dataset objects.
        copy: boolean, optional
            If copy is True, the data and affine arrays are copied, 
            elsewhere a view is taken.
        squeeze: boolean, optional
            If squeeze is True, the data array is squeeze on for
            dimensions above 3.
        world_space: string or None, optional
            An optional specification of the world space, to override
            that given by the image.

        Returns
        -------
        volume_img: VolumeImg object
            A VolumeImg object containing the data. The metadata is
            kept as much as possible in the metadata attribute.

        Notes
        ------
        The world space might not be correctly defined by the input
        object (in particular, when loading data from disk). In this
        case, you can correct it manually using the world_space keyword
        argument.

        For pynifti objects, the data is transposed.
    """
    if hasattr(obj, 'as_volume_img'):
        obj = obj.as_volume_img(copy=copy)
        if copy:
            obj = obj.__copy__()
        return obj

    elif isinstance(obj, basestring):
        if not os.path.exists(obj):
            raise ValueError("The file '%s' cannot be found" % obj)
        obj = nib.load(obj)
        copy = False
    
    if isinstance(obj, SpatialImage):
        data   = obj.get_data()
        affine = obj.get_affine()
        header = dict(obj.get_header())
        fname = obj.file_map['image'].filename
        if fname:
            header['filename'] = fname
    elif hasattr(obj, 'data') and hasattr(obj, 'sform') and \
                                            hasattr(obj, 'getVolumeExtent'):
        # Duck-types to a pynifti object
        data     = obj.data.T
        affine   = obj.sform
        header   = obj.header
        filename = obj.getFilename()
        if filename != '':
            header['filename'] = filename
    else:
        raise ValueError('Invalid type (%s) passed in: cannot convert %s to '
                    'VolumeImg' % (type(obj), obj))

    if world_space is None and header.get('sform_code', 0) == 4:
        world_space = 'mni152'

    data    = np.asanyarray(data)
    affine  = np.asanyarray(affine)
    if copy:
        data    = data.copy()
        affine  = affine.copy()

    if squeeze:
        # Squeeze the dimensions above 3
        shape = [val for index, val in enumerate(data.shape)
                     if val !=1 or index < 3]
        data = np.reshape(data, shape)
    
    return VolumeImg(data, affine, world_space, metadata=header)


def save(filename, obj):
    """ Save an nipy image object to a file.
    """
    obj = as_volume_img(obj, copy=False)
    hdr = nib.Nifti1Header()
    for key, value in obj.metadata.iteritems():
        if key in hdr:
            hdr[key] = value
    img = nib.Nifti1Image(obj.get_data(), 
                                   obj.affine,
                                   header=hdr)
    nib.save(img, filename)




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This example uses a different HRF for different event types
"""

import numpy as np

import matplotlib.pyplot as plt

from nipy.modalities.fmri import hrf
from nipy.modalities.fmri.utils import T, lambdify_t


# HRFs as functions of (symbolic) time
glover = hrf.glover(T)
afni = hrf.afni(T)

ta = [0,4,8,12,16]; tb = [2,6,10,14,18]
ba = 1; bb = -2
na = ba * sum([glover.subs(T, T - t) for t in ta])
nb = bb * sum([afni.subs(T, T - t) for t in tb])

nav = lambdify_t(na)
nbv = lambdify_t(nb)

t = np.linspace(0,30,200)
plt.plot(t, nav(t), c='r', label='Face')
plt.plot(t, nbv(t), c='b', label='Object')
plt.plot(t, nbv(t)+nav(t), c='g', label='Combined')

for t in ta:
    plt.plot([t,t],[0,ba*0.5],c='r')
for t in tb:
    plt.plot([t,t],[0,bb*0.5],c='b')
plt.plot([0,30], [0,0],c='#000000')
plt.legend()

plt.show()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Use Mayavi to visualize the structure of a VolumeImg
"""

from enthought.mayavi import mlab
import numpy as np

rand = np.random.RandomState(1)
data = rand.random_sample((5, 4, 4))

mlab.figure(1, fgcolor=(0, 0, 0), bgcolor=(1, 1, 1))
mlab.clf()

src = mlab.pipeline.scalar_field(data)
src.image_data.spacing = (0.5, 1, 0.7)
src.image_data.update_data()

mlab.pipeline.surface(src, opacity=0.4)
mlab.pipeline.surface(mlab.pipeline.extract_edges(src), color=(0, 0, 0))
mlab.pipeline.glyph(src, mode='cube', scale_factor=0.2, scale_mode='none')
mlab.savefig('volume_img.jpg')
mlab.show()





# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np
import numpy.linalg as npl

from .. import intvol, utils

from nose.tools import assert_equal, assert_raises
from numpy.testing import assert_array_equal, assert_almost_equal


def symnormal(p=10):
    M = np.random.standard_normal((p,p))
    return (M + M.T) / np.sqrt(2)


def randorth(p=10):
    """
    A random orthogonal matrix.
    """
    A = symnormal(p)
    return npl.eig(A)[1]


def box(shape, edges):
    data = np.zeros(shape)
    sl = []
    for i in range(len(shape)):
        sl.append(slice(edges[i][0], edges[i][1],1))
    data[sl] = 1
    return data.astype(np.int)


def randombox(shape):
    """
    Generate a random box, returning the box and the edge lengths
    """
    edges = [np.random.random_integers(0, shape[j], size=(2,))
             for j in range(len(shape))]

    for j in range(len(shape)):
        edges[j].sort()
        if edges[j][0] == edges[j][1]:
            edges[j][0] = 0; edges[j][1] = shape[j]/2+1
    return edges, box(shape, edges)


def elsym(edgelen, order=1):
    """
    Elementary symmetric polynomial of a given order
    """
    l = len(edgelen)
    if order == 0:
        return 1
    r = 0
    for v in utils.combinations(range(l), order):
        r += np.product([edgelen[vv] for vv in v])
    return r


def nonintersecting_boxes(shape):
    """
    The Lips's are supposed to be additive, so disjoint things
    should be additive. But, if they ALMOST intersect, different
    things get added to the triangulation.

    >>> b1 = np.zeros(40, np.int)
    >>> b1[:11] = 1
    >>> b2 = np.zeros(40, np.int)
    >>> b2[11:] = 1
    >>> (b1*b2).sum()
    0
    >>> c = np.indices((40,)).astype(np.float)
    >>> intvol.Lips1d(c, b1)
    array([  1.,  10.])
    >>> intvol.Lips1d(c, b2)
    array([  1.,  28.])
    >>> intvol.Lips1d(c, b1+b2)
    array([  1.,  39.])

    The function creates two boxes such that the 'dilated' box1 does not
    intersect with box2.  Additivity works in this case.
    """
    while True:
        edge1, box1 = randombox(shape)
        edge2, box2 = randombox(shape)

        diledge1 = [[max(ed[0]-1, 0), min(ed[1]+1, sh)] 
                    for ed, sh in zip(edge1, box1.shape)]

        dilbox1 = box(box1.shape, diledge1)

        if set(np.unique(dilbox1 + box2)).issubset([0,1]):
            break
    return box1, box2, edge1, edge2


def pts2dots(d, a, b, c):
    """ Convert point coordinates to dot products
    """
    D00 = np.dot(d, d)
    D01 = np.dot(d, a)
    D02 = np.dot(d, b)
    D03 = np.dot(d, c)
    D11 = np.dot(a, a)
    D12 = np.dot(a, b)
    D13 = np.dot(a, c)
    D22 = np.dot(b, b)
    D23 = np.dot(b, c)
    D33 = np.dot(c, c)
    return D00, D01, D02, D03, D11, D12, D13, D22, D23, D33


def pts2mu3_tet(d, a, b, c):
    """ Accept point coordinates for calling mu3tet
    """
    return intvol.mu3_tet(*pts2dots(d, a, b, c))


def wiki_tet_vol(d, a, b, c):
    # Wikipedia formula for generalized tetrahedron volume
    d, a, b, c = [np.array(e) for e in d, a, b, c]
    cp = np.cross((b-d),(c-d))
    v2t6 = np.dot((a-d), cp)
    return np.sqrt(v2t6) / 6.


def test_mu3tet():
    assert_equal(intvol.mu3_tet(0,0,0,0,1,0,0,1,0,1), 1./6)
    assert_equal(intvol.mu3_tet(0,0,0,0,0,0,0,0,0,0), 0)
    d = [2,2,2]
    a = [3,2,2]
    b = [2,3,2]
    c = [2,2,3]
    assert_equal(pts2mu3_tet(d, a, b, c), 1./6)
    assert_equal(wiki_tet_vol(d, a, b, c), 1./6)
    # This used to generate nan values
    assert_equal(intvol.mu3_tet(0,0,0,0,1,0,0,-1,0,1), 0)


def test_mu2tri():
    assert_equal(intvol.mu2_tri(0,0,0,1,0,1), 1./2)


def test_mu1tri():
    assert_equal(intvol.mu1_tri(0,0,0,1,0,1), 1+np.sqrt(2)/2)


def test_mu2tet():
    assert_equal(intvol.mu2_tet(0,0,0,0,1,0,0,1,0,1), (3./2 + np.sqrt(3./4))/2)


def pts2mu1_tet(d, a, b, c):
    """ Accept point coordinates for calling mu1_tet
    """
    return intvol.mu1_tet(*pts2dots(d, a, b, c))


def test_mu1_tet():
    res1 = pts2mu1_tet([2,2,2],[3,2,2],[2,3,2],[2,2,3])
    res2 = pts2mu1_tet([0,0,0],[1,0,0],[0,1,0],[0,0,1])
    assert_equal(res1, res2)
    assert_equal(intvol.mu1_tet(0,0,0,0,0,0,0,0,0,0), 0)
    # This used to generate nan values
    assert_equal(intvol.mu1_tet(0,0,0,0,1,0,0,-1,0,1), 0)


def test__mu1_tetface():
    # Test for out of range acos value sequences.  I'm ashamed to say I found
    # these sequences accidentally in a failing test with random numbers
    _mu1_tetface = intvol._mu1_tetface
    assert_almost_equal(_mu1_tetface(1, 0, 0, 10, 10, 0, 0, 20, 20, 40), 0)
    assert_almost_equal(_mu1_tetface(36, 0, 0, 18, 48, 0, 0, 1, 30,  63), 3)


def test_ec():
    for i in range(1, 4):
        _, box1 = randombox((40,)*i)
        f = {3:intvol.EC3d,
             2:intvol.EC2d,
             1:intvol.EC1d}[i]
        yield assert_almost_equal, f(box1), 1


def test_ec_disjoint():
    for i in range(1, 4):
        e = {3:intvol.EC3d,
             2:intvol.EC2d,
             1:intvol.EC1d}[i]
        box1, box2, _, _ = nonintersecting_boxes((40,)*i)
        assert_almost_equal(e(box1 + box2), e(box1) + e(box2))


def test_lips_wrapping():
    # Test that shapes touching the edge do not combine by wrapping
    b1 = np.zeros(40, np.int)
    b1[:11] = 1
    b2 = np.zeros(40, np.int)
    b2[11:] = 1
    # lines are disjoint
    assert_equal((b1*b2).sum(), 0)
    c = np.indices(b1.shape).astype(np.float)
    assert_array_equal(intvol.Lips1d(c, b1), (1, 10))
    assert_array_equal(intvol.Lips1d(c, b2), (1, 28))
    assert_array_equal(intvol.Lips1d(c, b1+b2), (1, 39.0))
    # 2D
    b1 = b1[:,None]
    b2 = b2[:,None]
    # boxes are disjoint
    assert_equal((b1*b2).sum(), 0)
    c = np.indices(b1.shape).astype(np.float)
    assert_array_equal(intvol.Lips2d(c, b1), (1, 10, 0))
    assert_array_equal(intvol.Lips2d(c, b2), (1, 28, 0))
    assert_array_equal(intvol.Lips2d(c, b1+b2), (1, 39.0, 0))
    # 3D
    b1 = b1[:,:,None]
    b2 = b2[:,:,None]
    assert_equal(b1.shape, (40,1,1))
    # boxes are disjoint
    assert_equal((b1*b2).sum(), 0)
    c = np.indices(b1.shape).astype(np.float)
    assert_array_equal(intvol.Lips3d(c, b1), (1, 10, 0, 0))
    assert_array_equal(intvol.Lips3d(c, b2), (1, 28, 0, 0))
    assert_array_equal(intvol.Lips3d(c, b1+b2), (1, 39.0, 0, 0))
    # Shapes which are squeezable should still return sensible answers
    # Test simple ones line / box / volume
    funcer = {1: (intvol.Lips1d, intvol.EC1d),
              2: (intvol.Lips2d, intvol.EC2d),
              3: (intvol.Lips3d, intvol.EC3d)}
    for box_shape, exp_ivs in [[(10,),(1,9)],
                               [(10,1),(1,9,0)],
                               [(1,10),(1,9,0)],
                               [(10,1,1), (1,9,0,0)],
                               [(1, 10, 1), (1,9,0,0)],
                               [(1, 1, 10), (1,9,0,0)]]:
        nd = len(box_shape)
        lips_func, ec_func = funcer[nd]
        c = np.indices(box_shape).astype(np.float)
        b = np.ones(box_shape, dtype=np.int)
        assert_array_equal(lips_func(c, b), exp_ivs)
        assert_equal(ec_func(b), exp_ivs[0])


def test_lips1_disjoint():
    phi = intvol.Lips1d
    box1, box2, edge1, edge2 = nonintersecting_boxes((30,))
    c = np.indices((30,)).astype(np.float)
    # Test N dimensional coordinates (N=10)
    d = np.random.standard_normal((10,)+(30,))
    # Test rotation causes no change in volumes
    U = randorth(p=6)[:1]
    e = np.dot(U.T, c.reshape((c.shape[0], np.product(c.shape[1:]))))
    e.shape = (e.shape[0],) +  c.shape[1:]

    assert_almost_equal(phi(c, box1 + box2), phi(c, box1) + phi(c, box2))
    assert_almost_equal(phi(d, box1 + box2), phi(d, box1) + phi(d, box2))
    assert_almost_equal(phi(e, box1 + box2), phi(e, box1) + phi(e, box2))
    assert_almost_equal(phi(e, box1 + box2), phi(c, box1 + box2))
    assert_almost_equal(phi(e, box1 + box2),
                        (np.array(
                            [elsym([e[1]-e[0]-1
                                    for e in edge1], i) for i in range(2)]) +
                        np.array(
                            [elsym([e[1]-e[0]-1
                                    for e in edge2], i) for i in range(2)])))
    assert_raises(ValueError, phi, c[...,None], box1)


def test_lips2_disjoint():
    phi = intvol.Lips2d
    box1, box2, edge1, edge2 = nonintersecting_boxes((40,40))
    c = np.indices((40,40)).astype(np.float)
    # Test N dimensional coordinates (N=10)
    d = np.random.standard_normal((10,40,40))
    # Test rotation causes no change in volumes
    U = randorth(p=6)[0:2]
    e = np.dot(U.T, c.reshape((c.shape[0], np.product(c.shape[1:]))))
    e.shape = (e.shape[0],) +  c.shape[1:]
    assert_almost_equal(phi(c, box1 + box2),
                        phi(c, box1) + phi(c, box2))
    assert_almost_equal(phi(d, box1 + box2),
                        phi(d, box1) + phi(d, box2))
    assert_almost_equal(phi(e, box1 + box2),
                        phi(e, box1) + phi(e, box2))
    assert_almost_equal(phi(e, box1 + box2), phi(c, box1 + box2))
    assert_almost_equal(phi(e, box1 + box2),
                        np.array([elsym([e[1]-e[0]-1 for e in edge1], i)
                                   for i in range(3)]) +
                        np.array([elsym([e[1]-e[0]-1 for e in edge2], i)
                                  for i in range(3)])
                       )
    assert_raises(ValueError, phi, c[...,None], box1)
    assert_raises(ValueError, phi, c[:,:,1], box1)


def test_lips3_disjoint():
    phi = intvol.Lips3d
    box1, box2, edge1, edge2 = nonintersecting_boxes((40,)*3)
    c = np.indices((40,)*3).astype(np.float)
    # Test N dimensional coordinates (N=10)
    d = np.random.standard_normal((10,40,40,40))
    # Test rotation causes no change in volumes
    U = randorth(p=6)[0:3]
    e = np.dot(U.T, c.reshape((c.shape[0], np.product(c.shape[1:]))))
    e.shape = (e.shape[0],) +  c.shape[1:]

    assert_almost_equal(phi(c, box1 + box2), phi(c, box1) + phi(c, box2))
    assert_almost_equal(phi(d, box1 + box2), phi(d, box1) + phi(d, box2))
    assert_almost_equal(phi(e, box1 + box2), phi(e, box1) + phi(e, box2))
    assert_almost_equal(phi(e, box1 + box2), phi(c, box1 + box2))
    assert_almost_equal(
        phi(e, box1 + box2),
        (np.array([elsym([e[1]-e[0]-1 for e in edge1], i) for i in range(4)]) +
         np.array([elsym([e[1]-e[0]-1 for e in edge2], i) for i in range(4)])))
    assert_raises(ValueError, phi, c[...,None], box1)
    assert_raises(ValueError, phi, c[:,:,:,1], box1)


def test_lips3_nans():
    # These boxes caused nans in the Lips3 disjoint box tests
    phi = intvol.Lips3d
    box1 = np.zeros((40,40,40), dtype=np.int)
    box2 = box1.copy()
    box1[23:30,22:32,9:13] = 1
    box2[7:22,0,8:17] = 1
    c = np.indices(box1.shape).astype(np.float)
    assert_array_equal(np.isnan(phi(c, box2)), False)
    U = randorth(p=6)[0:3]
    e = np.dot(U.T, c.reshape((c.shape[0], np.product(c.shape[1:]))))
    e.shape = (e.shape[0],) +  c.shape[1:]
    assert_array_equal(np.isnan(phi(e, box1 + box2)), False)


def test_slices():
    # Slices have EC 1...
    e = intvol.EC3d
    p = intvol.Lips3d
    m = np.zeros((40,)*3, np.int)
    D = np.indices(m.shape).astype(np.float)
    m[10,10,10] = 1
    yield assert_almost_equal, e(m), 1
    yield assert_almost_equal, p(D,m), [1,0,0,0]

    m = np.zeros((40,)*3, np.int)
    m[10,10:14,10] = 1
    yield assert_almost_equal, e(m), 1
    yield assert_almost_equal, p(D,m), [1,3,0,0]

    m = np.zeros((40,)*3, np.int)
    m[10,10:14,9:15] = 1
    yield assert_almost_equal, e(m), 1
    yield assert_almost_equal, p(D,m), [1,8,15,0]


def test_ec_wrapping():
    # Test wrapping for EC1 calculation
    assert_equal(intvol.EC1d(np.ones((6,), dtype=np.int)), 1)
    box1 = np.array([1, 1, 0, 1, 1, 1], dtype=np.int)
    assert_equal(intvol.EC1d(box1), 2)
    # 2D
    box1 = np.zeros((3,6), dtype=np.int)
    box1[1] = 1
    assert_equal(intvol.EC2d(box1), 1)
    box1[1, 3] = 0
    assert_equal(intvol.EC2d(box1), 2)
    # 3D
    box1 = np.zeros((3,6,3), dtype=np.int)
    box1[1, :, 1] = 1
    assert_equal(intvol.EC3d(box1), 1)
    box1[1, 3, 1] = 0
    assert_equal(intvol.EC3d(box1), 2)



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Script to run nose with coverage reporting without boilerplate params.

Usage:
  sneeze test_coordinate_system.py

Coverage will be reported on the module extracted from the test file
name by removing the 'test_' prefix and '.py' suffix.  In the above
example, we'd get the coverage on the coordinate_system module.  The
test file is searched for an import statement containing the module
name.

The nose command would look like this:

nosetests -sv --with-coverage --cover-package=nipy.core.reference.coordinate_system test_coordinate_system.py

"""

import os
import sys
import nose
from optparse import OptionParser

usage_doc = "usage: sneeze test_module.py"

def find_pkg(pkg):
    test_file = os.path.basename(pkg)
    module = os.path.splitext(test_file)[0] # remove '.py' extension
    module = module.split('test_')[1] # remove 'test_' prefix
    
    cover_pkg = None
    fp = open(pkg, 'r')
    for line in fp:
        if line.startswith('from') or line.startswith('import'):
            # remove keywords from import line
            imptline = line.replace('import', '')
            imptline = imptline.replace('from', '')
            imptline = imptline.replace('as', '')
            # split and rejoin with periods.  This will remove
            # whitespace and join together a complete namespace.
            imptline = imptline.split()
            imptline = '.'.join(imptline)
            try:
                # Find index that immediately follows the module we care about
                index = imptline.index(module) 
                index += len(module)
                cover_pkg = imptline[:index] 
                break
            except ValueError:
                pass
    fp.close()
    return cover_pkg, module

def run_nose(cover_pkg, test_file, dry_run=False):
    cover_arg = '--cover-package=%s' % cover_pkg
    sys.argv += ['-sv', '--with-coverage', cover_arg]
    # Print out command for user feedback and debugging
    cmd = 'nosetests -sv --with-coverage %s %s' % (cover_arg, test_file)
    print cmd
    if dry_run:
        return cmd
    else:
        print
        nose.run()


def main():
    description = __doc__.splitlines()[0]
    parser = OptionParser(usage=usage_doc, description=description)
    parser.add_option('-n', '--dry-run', action="store_true", dest="dry_run",
                      help='Return generated nose command without executing.')
    options, args = parser.parse_args()
    if options.dry_run:
        # If we don't remove the -n option, it breaks the execution of nose.
        sys.argv.remove('-n')

    try:
        test_file = args[0]
        cover_pkg, module = find_pkg(test_file)
        if cover_pkg:
            run_nose(cover_pkg, test_file, dry_run=options.dry_run)
        else:
            raise ValueError('Unable to find module %s imported in test file %s'
                             % (module, test_file))
    except IndexError:
        parser.print_help()


if __name__ == '__main__':
    main()





from __future__ import with_statement
from os.path import exists
import numpy as np
from nibabel import Nifti1Image, save
from numpy.testing import assert_equal
from ...utils.simul_multisubject_fmri_dataset import surrogate_3d_dataset
from ..parcel_io import (mask_parcellation, fixed_parcellation,
                         parcellation_based_analysis)
from ..hierarchical_parcellation import hparcel
from ..discrete_domain import grid_domain_from_shape
from nibabel.tmpdirs import InTemporaryDirectory


def test_mask_parcel():
    """ Test that mask parcellation performs correctly
    """
    n_parcels = 20
    shape = (10, 10, 10)
    mask_image = Nifti1Image(np.ones(shape), np.eye(4))
    wim = mask_parcellation(mask_image, n_parcels)
    assert_equal(np.unique(wim.get_data()), np.arange(n_parcels))


def test_mask_parcel_multi_subj():
    """ Test that mask parcellation performs correctly
    """
    rng = np.random.RandomState(0); 
    n_parcels = 20
    shape = (10, 10, 10)
    n_subjects = 5
    mask_images = []
    with InTemporaryDirectory():
        for subject in range(n_subjects):
            path = 'mask%s.nii' % subject
            save(Nifti1Image((rng.rand(*shape) > .1).astype('u8'),
                             np.eye(4)), path)
            mask_images.append(path)

        wim = mask_parcellation(mask_images, n_parcels)
        assert_equal(np.unique(wim.get_data()), np.arange(n_parcels))


def test_parcel_intra_from_3d_image():
    """Test that a parcellation is generated, starting from an input 3D image
    """
    # Generate an image
    shape = (10, 10, 10)
    n_parcel, nn, mu = 10, 6, 1.
    mask_image = Nifti1Image(np.ones(shape), np.eye(4))
    with InTemporaryDirectory() as dir_context:
        surrogate_3d_dataset(mask=mask_image, out_image_file='image.nii')

        #run the algo 
        for method in ['ward', 'kmeans', 'gkm']:
            osp = fixed_parcellation(mask_image, ['image.nii'], n_parcel, nn,
                                     method, dir_context, mu)
            result = 'parcel_%s.nii' % method
            assert exists(result)
            assert_equal(osp.k, n_parcel)


def test_parcel_intra_from_3d_images_list():
    """Test that a parcellation is generated, starting from a list of 3D images
    """
    # Generate an image
    shape = (10, 10, 10)
    n_parcel, nn, mu = 10, 6, 1.
    method = 'ward'
    mask_image = Nifti1Image(np.ones(shape), np.eye(4))

    with InTemporaryDirectory() as dir_context:
        data_image = ['image_%d.nii' % i for i in range(5)]
        for datim in data_image:
            surrogate_3d_dataset(mask=mask_image, out_image_file=datim)

        #run the algo
        osp = fixed_parcellation(mask_image, data_image, n_parcel, nn,
                                 method, dir_context, mu)
        assert exists('parcel_%s.nii' % method)
        assert_equal(osp.k, n_parcel)


def test_parcel_intra_from_4d_image():
    """Test that a parcellation is generated, starting from a 4D image
    """
    # Generate an image
    shape = (10, 10, 10)
    n_parcel, nn, mu = 10, 6, 1.
    method = 'ward'
    mask_image = Nifti1Image(np.ones(shape), np.eye(4))
    with InTemporaryDirectory() as dir_context:
        surrogate_3d_dataset(n_subj=10, mask=mask_image, 
                             out_image_file='image.nii')    
        osp = fixed_parcellation(mask_image, ['image.nii'], n_parcel, nn,
                                 method, dir_context, mu)
        assert exists('parcel_%s.nii' % method)
        assert_equal(osp.k, n_parcel)

def test_parcel_based_analysis():
    # Generate an image
    shape = (7, 8, 4)
    n_subj = 5
    n_parcel, nn, mu = 10, 6, 1.

    with InTemporaryDirectory() as dir_context:
        data_image = ['image_%d.nii' % i for i in range(5)]
        for datim in data_image:
            surrogate_3d_dataset(shape=shape, out_image_file=datim)
        ldata = np.random.randn(n_subj, np.prod(shape), 1)
        domain = grid_domain_from_shape(shape)
        parcels = hparcel(domain, ldata, n_parcel, mu=3.0)
        prfx = parcellation_based_analysis(
            parcels, data_image, test_id='one_sample', rfx_path='prfx.nii',
            condition_id='', swd=dir_context)
        assert exists('prfx.nii')
        assert prfx.max() < 10
        assert prfx.min() > - 10

if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from StringIO import StringIO

import numpy as np

import scipy.stats

from ...formula.formulae import Factor, make_recarray
from ....utils.matrices import matrix_rank
from ..regression import OLSModel

from nipy.testing import (assert_equal, assert_almost_equal)


data = """0.0      1      1      1
    2.0      1      1      2
    1.0      1      1      3
    3.0      1      1      4
    0.0      1      1      5
    2.0      1      1      6
    0.0      1      1      7
    5.0      1      1      8
    6.0      1      1      9
    8.0      1      1     10
    2.0      1      2      1
    4.0      1      2      2
    7.0      1      2      3
   12.0      1      2      4
   15.0      1      2      5
    4.0      1      2      6
    3.0      1      2      7
    1.0      1      2      8
    5.0      1      2      9
   20.0      1      2     10
   15.0      1      3      1
   10.0      1      3      2
    8.0      1      3      3
    5.0      1      3      4
   25.0      1      3      5
   16.0      1      3      6
    7.0      1      3      7
   30.0      1      3      8
    3.0      1      3      9
   27.0      1      3     10
    0.0      2      1      1
    1.0      2      1      2
    1.0      2      1      3
    0.0      2      1      4
    4.0      2      1      5
    2.0      2      1      6
    7.0      2      1      7
    4.0      2      1      8
    0.0      2      1      9
    3.0      2      1     10
    5.0      2      2      1
    3.0      2      2      2
    2.0      2      2      3
    0.0      2      2      4
    1.0      2      2      5
    1.0      2      2      6
    3.0      2      2      7
    6.0      2      2      8
    7.0      2      2      9
    9.0      2      2     10
   10.0      2      3      1
    8.0      2      3      2
   12.0      2      3      3
    3.0      2      3      4
    7.0      2      3      5
   15.0      2      3      6
    4.0      2      3      7
    9.0      2      3      8
    6.0      2      3      9
    1.0      2      3     10"""

# Load some data in -- the above data
# is an example from 
# "Applied Linear Statistical Models" that can be found
# The number of Days in a hospital stay are recorded
# based on the Duration of dialysis treatment
# and the Weight of the patient. These
# two variables are described categorically
# as Duration (1 or 2), Weight (1, 2 or 3)
#
# You can find another copy of the data at
#
# http://www-stat.stanford.edu/~jtaylo/courses/stats191/data/kidney.table

D = []
for row in StringIO(data):
    D.append(map(float, row.split()))
D = make_recarray(D, ['Days', 'Duration', 'Weight', 'ID'])

# Create the categorical regressors, known as Factors

f1 = Factor('Duration', [1,2])
f2 = Factor('Weight', [1,2,3])

twoway = f1 * f2

# The columns of X are 0-1 indicator columns,
# return_float = False yields a recarray 
# with interpretable names

def test_names():
    # Check that the design column names are what we expect
    X = twoway.design(D, return_float=False)
    assert_equal(set(X.dtype.names), set(('Duration_1*Weight_1', 'Duration_1*Weight_2', 'Duration_1*Weight_3', 'Duration_2*Weight_1', 'Duration_2*Weight_2', 'Duration_2*Weight_3')))

# If we ask for contrasts, the resulting matrix is
# of dtype np.float

contrasts = {'Duration': f1.main_effect,
             'Weight': f2.main_effect,
             'Interaction': f1.main_effect * f2.main_effect,
             'Duration1': f1.get_term(1)}
X, cons = twoway.design(D, contrasts=contrasts)
Y = D['Days']


# Fit the model

beta = np.dot(np.linalg.pinv(X), Y)
XTXinv = np.linalg.pinv(np.dot(X.T, X))
resid = Y - np.dot(X, beta)
df_resid = (X.shape[0] - matrix_rank(X)) # residual degrees of freedom
sigmasq = (resid**2).sum() / df_resid

SS = {}
MS = {}
F = {}
df = {}
p = {}
for n, c in cons.items():
    cbeta = np.dot(c, beta)
    cov_cbeta = np.dot(c, np.dot(XTXinv, c.T))
    if c.ndim > 1:
        df[n] = c.shape[0]
        SS[n] = np.dot(cbeta, np.dot(np.linalg.pinv(cov_cbeta), cbeta))
        MS[n] = SS[n] / df[n]
        F[n] = MS[n] / sigmasq
    else:
        df[n] = 1
        SS[n] = (cbeta**2).sum() / cov_cbeta
        MS[n] = SS[n] / df[n]
        F[n] = MS[n] / sigmasq
    p[n] = scipy.stats.f.sf(F[n], df[n], df_resid)

routput = \
"""
Output of R:
-----------


> anova(lm(Days~Duration*Weight, X))
Analysis of Variance Table

Response: Days
                Df  Sum Sq Mean Sq F value    Pr(>F)    
Duration         1  209.07  209.07  7.2147  0.009587 ** 
Weight           2  760.43  380.22 13.1210 2.269e-05 ***
Duration:Weight  2  109.03   54.52  1.8813  0.162240    
Residuals       54 1564.80   28.98                      
---
"""


def test_Ragreement():
    # This code would fit the two-way ANOVA model in R


    # X = read.table('http://www-stat.stanford.edu/~jtaylo/courses/stats191/data/kidney.table', header=T)
    # names(X)
    # X$Duration = factor(X$Duration)
    # X$Weight = factor(X$Weight)
    # lm(Days~Duration*Weight, X)
    # A = anova(lm(Days~Duration*Weight, X))


    # rA = rpy.r('A')
    rA = {'Df': [1, 2, 2, 54],
     'F value': [7.2147239263803673, 13.120973926380339, 1.8813266871165633, np.nan],
     'Mean Sq': [209.06666666666663,
                 380.21666666666584,
                 54.51666666666663,
                 28.977777777777778],
     'Pr(>F)': [0.0095871255601553771,
                2.2687781292164585e-05,
                0.16224035152442268,
                np.nan],
     'Sum Sq': [209.06666666666663,
                760.43333333333169,
                109.03333333333326,
                1564.8]}

    # rn = rpy.r('rownames(A)')
    rn= ['Duration', 'Weight', 'Duration:Weight', 'Residuals']

    pairs = [(rn.index('Duration'), 'Duration'),
             (rn.index('Weight'), 'Weight'),
             (rn.index('Duration:Weight'), 'Interaction')]

    for i, j in pairs:
        assert_almost_equal(F[j], rA['F value'][i])
        assert_almost_equal(p[j], rA['Pr(>F)'][i])
        assert_almost_equal(MS[j], rA['Mean Sq'][i])
        assert_almost_equal(df[j], rA['Df'][i])
        assert_almost_equal(SS[j], rA['Sum Sq'][i])


def test_scipy_stats():
    # Using scipy.stats.models

    X, cons = twoway.design(D, contrasts=contrasts)
    Y = D['Days']
    m = OLSModel(X)
    f = m.fit(Y)

    F_m = {}
    df_m = {}
    p_m = {}

    for n, c in cons.items():
        r = f.Fcontrast(c)
        F_m[n] = r.F
        df_m[n] = r.df_num
        p_m[n] = scipy.stats.f.sf(F_m[n], df_m[n], r.df_den)
        assert_almost_equal(F[n], F_m[n])
        assert_almost_equal(df[n], df_m[n])
        assert_almost_equal(p[n], p_m[n])



#!/usr/bin/env python


#
# Test BLAS 3
#


from numpy.testing import assert_almost_equal
import numpy as np
from .. import (blas_dgemm, blas_dsymm, blas_dtrmm, 
                blas_dtrsm, blas_dsyrk, blas_dsyr2k)

n1 = 10
n2 = 13

    
def test_dgemm():
    A = np.random.rand(n1,n2)
    B = np.random.rand(n2,n1)
    C = np.random.rand(n1,n1)
    C2 = np.random.rand(n2,n2)
    alpha = np.double(np.random.rand(1))
    beta = np.double(np.random.rand(1))
    # Test: A*B
    Dgold = alpha*np.dot(A,B) + beta*C
    D = blas_dgemm(0, 0, alpha, A, B, beta, C)
    assert_almost_equal(Dgold, D) 
    # Test: A^t B^t
    Dgold = alpha*np.dot(A.T,B.T) + beta*C2
    D = blas_dgemm(1, 1, alpha, A, B, beta, C2)
    assert_almost_equal(Dgold, D) 

def test_dsymm():
    A = np.random.rand(n1,n1)
    A = A + A.T
    B = np.random.rand(n1,n2)
    C = np.random.rand(n1,n2)
    B2 = np.random.rand(n2,n1)
    C2 = np.random.rand(n2,n1)
    alpha = np.double(np.random.rand(1))
    beta = np.double(np.random.rand(1))
    # Test: A*B
    Dgold = alpha*np.dot(A,B) + beta*C
    D = blas_dsymm(0, 0, alpha, A, B, beta, C)
    assert_almost_equal(Dgold, D)
    D = blas_dsymm(0, 1, alpha, A, B, beta, C)
    assert_almost_equal(Dgold, D)
    # Test: B*A
    Dgold = alpha*np.dot(B2,A) + beta*C2
    D = blas_dsymm(1, 0, alpha, A, B2, beta, C2)
    assert_almost_equal(Dgold, D)
    D = blas_dsymm(1, 1, alpha, A, B2, beta, C2)
    assert_almost_equal(Dgold, D)

def _test_dtrXm(A, U, L, B, alpha, blasfn):
    # Test: U*B
    Dgold = alpha*np.dot(U,B) 
    D = blasfn(0, 0, 0, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: B*U
    Dgold = alpha*np.dot(B,U)
    D = blasfn(1, 0, 0, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: U'*B
    Dgold = alpha*np.dot(U.T,B) 
    D = blasfn(0, 0, 1, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: B*U'
    Dgold = alpha*np.dot(B,U.T)
    D = blasfn(1, 0, 1, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: L*B
    Dgold = alpha*np.dot(L,B) 
    D = blasfn(0, 1, 0, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: B*L
    Dgold = alpha*np.dot(B,L)
    D = blasfn(1, 1, 0, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: L'*B
    Dgold = alpha*np.dot(L.T,B) 
    D = blasfn(0, 1, 1, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: B*L'
    Dgold = alpha*np.dot(B,L.T)
    D = blasfn(1, 1, 1, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: U*B
    Dgold = alpha*np.dot(U,B) 
    D = blasfn(0, 0, 0, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: B*U
    Dgold = alpha*np.dot(B,U)
    D = blasfn(1, 0, 0, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: U'*B
    Dgold = alpha*np.dot(U.T,B) 
    D = blasfn(0, 0, 1, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: B*U'
    Dgold = alpha*np.dot(B,U.T)
    D = blasfn(1, 0, 1, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: L*B
    Dgold = alpha*np.dot(L,B) 
    D = blasfn(0, 1, 0, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: B*L
    Dgold = alpha*np.dot(B,L)
    D = blasfn(1, 1, 0, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: L'*B
    Dgold = alpha*np.dot(L.T,B) 
    D = blasfn(0, 1, 1, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)
    # Test: B*L'
    Dgold = alpha*np.dot(B,L.T)
    D = blasfn(1, 1, 1, 0, alpha, A, B)
    assert_almost_equal(Dgold, D)

def test_dtrmm():
    A = np.random.rand(n1,n1)
    U = np.triu(A)
    L = np.tril(A)
    B = np.random.rand(n1,n1)
    alpha = np.double(np.random.rand(1))
    _test_dtrXm(A, U, L, B, alpha, blas_dtrmm)

def test_dtrsm():
    A = np.random.rand(n1,n1)
    U = np.linalg.inv(np.triu(A))
    L = np.linalg.inv(np.tril(A))
    B = np.random.rand(n1,n1)
    alpha = np.double(np.random.rand(1))
    _test_dtrXm(A, U, L, B, alpha, blas_dtrsm)
    
def test_dsyrk(): 
    A = np.random.rand(n1,n1)
    C = np.random.rand(n1,n1)
    alpha = np.double(np.random.rand(1))
    beta = np.double(np.random.rand(1))
    # Test A*A'
    U = np.triu(blas_dsyrk(0, 0, alpha, A, beta, C))
    L = np.tril(blas_dsyrk(1, 0, alpha, A, beta, C))
    Dgold = alpha*np.dot(A, A.T) + beta*C
    Ugold = np.triu(Dgold)
    Lgold = np.tril(Dgold)
    assert_almost_equal(Ugold, U)
    assert_almost_equal(Lgold, L)
    # Test A'*A
    U = np.triu(blas_dsyrk(0, 1, alpha, A, beta, C))
    L = np.tril(blas_dsyrk(1, 1, alpha, A, beta, C))
    Dgold = alpha*np.dot(A.T, A) + beta*C
    Ugold = np.triu(Dgold)
    Lgold = np.tril(Dgold)
    assert_almost_equal(Ugold, U)
    assert_almost_equal(Lgold, L)

def test_dsyr2k(): 
    A = np.random.rand(n1,n1)
    B = np.random.rand(n1,n1)
    C = np.random.rand(n1,n1)
    alpha = np.double(np.random.rand(1))
    beta = np.double(np.random.rand(1))
    # Test A*B' + B*A'
    U = np.triu(blas_dsyr2k(0, 0, alpha, A, B, beta, C))
    L = np.tril(blas_dsyr2k(1, 0, alpha, A, B, beta, C))
    Dgold = alpha*(np.dot(A,B.T) + np.dot(B,A.T)) + beta*C
    Ugold = np.triu(Dgold)
    Lgold = np.tril(Dgold)
    assert_almost_equal(Ugold, U)
    assert_almost_equal(Lgold, L)
    # Test A'*B + B'*A
    U = np.triu(blas_dsyr2k(0, 1, alpha, A, B, beta, C))
    L = np.tril(blas_dsyr2k(1, 1, alpha, A, B, beta, C))
    Dgold = alpha*(np.dot(A.T,B) + np.dot(B.T,A)) + beta*C
    Ugold = np.triu(Dgold)
    Lgold = np.tril(Dgold)
    assert_almost_equal(Ugold, U)
    assert_almost_equal(Lgold, L)


if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import warnings

import numpy as np

import nibabel as nib

from .. import image
from ..image import Image, iter_axis, is_image, rollimg
from ...api import parcels, data_generator, write_data
from ...reference.coordinate_system import CoordinateSystem
from ...reference.coordinate_map import AffineTransform, AxisError

from nose.tools import (assert_true, assert_false, assert_equal,
                        assert_not_equal, assert_raises)

from numpy.testing import (assert_array_almost_equal, assert_almost_equal,
                           assert_array_equal)

def setup():
    # Suppress warnings during tests to reduce noise
    warnings.simplefilter("ignore")

def teardown():
    # Clear list of warning filters
    warnings.resetwarnings()

_data = np.arange(24).reshape((4,3,2))
gimg = Image(_data, AffineTransform('ijk', 'xyz', np.eye(4)))


def test_init():
    data = gimg.get_data()
    new = Image(data, gimg.coordmap)
    assert_array_almost_equal(gimg.get_data(), new.get_data())
    assert_equal(new.coordmap, gimg.coordmap)
    assert_raises(TypeError, Image)
    assert_raises(TypeError, Image, data)


def test_maxmin_values():
    y = gimg.get_data()
    assert_equal(y.shape, tuple(gimg.shape))
    assert_equal(y.max(), 23)
    assert_equal(y.min(), 0.0)


def test_slice_plane():
    x = gimg[1]
    assert_equal(x.shape, gimg.shape[1:])


def test_slice_block():
    x = gimg[1:3]
    assert_equal(x.shape, (2,) + tuple(gimg.shape[1:]))


def test_slice_step():
    s = slice(0,4,2)
    x = gimg[s]
    assert_equal(x.shape, (2,) + tuple(gimg.shape[1:]))
    x = gimg[0:4:2]
    assert_equal(x.shape, (2,) + tuple(gimg.shape[1:]))


def test_slice_type():
    s = slice(0, gimg.shape[0])
    x = gimg[s]
    assert_equal(x.shape, gimg.shape)
    x = gimg[0:]
    assert_equal(x.shape, gimg.shape)


def test_slice_steps():
    dim0, dim1, dim2 = gimg.shape
    slice_z = slice(0, dim0, 2)
    slice_y = slice(0, dim1, 2)
    slice_x = slice(0, dim2, 2)
    x = gimg[slice_z, slice_y, slice_x]
    newshape = tuple(np.floor((np.array(gimg.shape) - 1)/2) + 1)
    assert_equal(x.shape, newshape)
    x = gimg[0:dim0:2,0:dim1:2,0:dim2:2]
    assert_equal(x.shape, newshape)


def test_get_data():
    # get_data always returns an array
    x = gimg.get_data()
    assert_true(isinstance(x, np.ndarray))
    assert_equal(x.shape, gimg.shape)
    assert_equal(x.ndim, gimg.ndim)


def test_generator():
    # User iter_axis to return slices
    gen = iter_axis(gimg, axis=0)
    for img_slice in gen:
        assert_equal(img_slice.shape, (3,2))


def test_iter():
    for img_slice in image.iter_axis(gimg, 0):
        assert_equal(img_slice.shape, (3,2))
    tmp = np.zeros(gimg.shape)
    write_data(tmp, enumerate(iter_axis(gimg, 0, asarray=True)))
    assert_array_almost_equal(tmp, gimg.get_data())
    tmp = np.zeros(gimg.shape)
    g = iter_axis(gimg, 0, asarray=True)
    write_data(tmp, enumerate(g))
    assert_array_almost_equal(tmp, gimg.get_data())


def test_parcels1():
    parcelmap = gimg.get_data().astype(np.int32)
    test = np.zeros(parcelmap.shape)
    v = 0
    for i, d in data_generator(test, parcels(parcelmap)):
        v += d.shape[0]
    assert_equal(v, np.product(test.shape))


def test_parcels3():
    rho = gimg[0]
    parcelmap = rho.get_data().astype(np.int32)
    labels = np.unique(parcelmap)
    test = np.zeros(rho.shape)
    v = 0
    for i, d in data_generator(test, parcels(parcelmap, labels=labels)):
        v += d.shape[0]
    yield assert_equal, v, np.product(test.shape)


def test_slicing_returns_image():
    data = np.ones((2,3,4))
    img = Image(data, AffineTransform('kji', 'zyx', np.eye(4)))
    assert_true(isinstance(img, Image))
    assert_equal(img.ndim, 3)
    # 2D slice
    img2D = img[:,:,0]
    assert_true(isinstance(img2D, Image))
    assert_equal(img2D.ndim, 2)
    # 1D slice
    img1D = img[:,0,0]
    assert_true(isinstance(img1D, Image))
    assert_equal(img1D.ndim, 1)


class ArrayLikeObj(object):
    """The data attr in Image is an array-like object.
    Test the array-like interface that we'll expect to support."""
    def __init__(self):
        self._data = np.ones((2,3,4))

    @property
    def shape(self):
        return self._data.shape

    def __array__(self):
        return self._data


def test_ArrayLikeObj():
    obj = ArrayLikeObj()
    # create simple coordmap
    xform = np.eye(4)
    coordmap = AffineTransform.from_params('xyz', 'ijk', xform)
    # create image form array-like object and coordmap
    img = image.Image(obj, coordmap)
    assert_equal(img.ndim, 3)
    assert_equal(img.shape, (2,3,4))
    assert_array_almost_equal(img.get_data(), 1)
    # Test that the array stays with the image, so we can assign the array
    # in-place, at least in this case
    img.get_data()[:] = 4
    assert_array_equal(img.get_data(), 4)


def test_defaults_ND():
    for arr_shape, in_names, out_names in (
        ((2,3), 'kj', 'yz'),
        ((2,3,4), 'ijk', 'zyx'),
        ((2,3,4,5), 'hijk', 'zyxt')):
        data = np.ones(arr_shape)
        ndim = len(arr_shape)
        img = Image(data, AffineTransform(in_names, out_names, np.eye(ndim+1)))
        assert_true(isinstance(img._data, np.ndarray))
        assert_equal(img.ndim, len(arr_shape))
        assert_equal(img.shape, arr_shape)
        assert_equal(img.affine.shape, (img.ndim+1, img.ndim+1))
        assert_true(img.affine.diagonal().all())
        # img.header deprecated, when removed, test will raise Error
        assert_raises(AttributeError, getattr, img, 'header')


def test_header():
    # Property header interface deprecated
    arr = np.arange(24).reshape((2,3,4))
    coordmap = AffineTransform.from_params('xyz', 'ijk', np.eye(4))
    header = nib.Nifti1Header()
    img = Image(arr, coordmap, metadata={'header': header})
    assert_equal(img.metadata['header'], header)
    # This interface deprecated
    assert_equal(img.header, header)
    hdr2 = nib.Nifti1Header()
    hdr2['descrip'] = 'from fullness of heart'
    assert_not_equal(img.header, hdr2)
    img.header = hdr2
    assert_equal(img.header, hdr2)


def test_from_image():
    # from_image classmethod copies
    arr = np.arange(24).reshape((2,3,4))
    coordmap = AffineTransform.from_params('xyz', 'ijk', np.eye(4))
    img = Image(arr, coordmap, metadata={'field': 'value'})
    img2 = Image.from_image(img)
    assert_array_equal(img.get_data(), img2.get_data())
    assert_equal(img.coordmap, img2.coordmap)
    assert_equal(img.metadata, img2.metadata)
    assert_false(img.metadata is img2.metadata)
    # optional inputs - data
    arr2 = arr + 10
    new = Image.from_image(img, arr2)
    assert_array_almost_equal(arr2, new.get_data())
    assert_equal(new.coordmap, coordmap)
    new = Image.from_image(img, data=arr2)
    assert_array_almost_equal(arr2, new.get_data())
    assert_equal(new.coordmap, coordmap)
    # optional inputs - coordmap
    coordmap2 = AffineTransform.from_params('pqr', 'ijk', np.eye(4))
    new = Image.from_image(img, arr2, coordmap2)
    assert_array_almost_equal(arr2, new.get_data())
    assert_equal(new.coordmap, coordmap2)
    new = Image.from_image(img, coordmap=coordmap2)
    assert_array_almost_equal(arr, new.get_data())
    assert_equal(new.coordmap, coordmap2)
    # Optional inputs - metadata
    assert_equal(new.metadata, img.metadata)
    another_meta = {'interesting': 'information'}
    new = Image.from_image(img, arr2, coordmap2, another_meta)
    assert_array_almost_equal(arr2, new.get_data())
    assert_equal(another_meta, new.metadata)


def test_synchronized_order():
    data = np.random.standard_normal((3,4,7,5))
    im = Image(data, AffineTransform.from_params('ijkl', 'xyzt', np.diag([1,2,3,4,1])))
    im_scrambled = im.reordered_axes('iljk').reordered_reference('xtyz')
    im_unscrambled = image.synchronized_order(im_scrambled, im)
    assert_equal(im_unscrambled.coordmap, im.coordmap)
    assert_almost_equal(im_unscrambled.get_data(), im.get_data())
    assert_equal(im_unscrambled, im)
    assert_true(im_unscrambled == im)
    assert_false(im_unscrambled != im)
    # the images don't have to be the same shape
    data2 = np.random.standard_normal((3,11,9,4))
    im2 = Image(data2, AffineTransform.from_params('ijkl', 'xyzt', np.diag([1,2,3,4,1])))
    im_scrambled2 = im2.reordered_axes('iljk').reordered_reference('xtyz')
    im_unscrambled2 = image.synchronized_order(im_scrambled2, im)
    assert_equal(im_unscrambled2.coordmap, im.coordmap)
    # or the same coordmap
    data3 = np.random.standard_normal((3,11,9,4))
    im3 = Image(data3, AffineTransform.from_params('ijkl', 'xyzt', np.diag([1,9,3,-2,1])))
    im_scrambled3 = im3.reordered_axes('iljk').reordered_reference('xtyz')
    im_unscrambled3 = image.synchronized_order(im_scrambled3, im)
    assert_equal(im_unscrambled3.axes, im.axes)
    assert_equal(im_unscrambled3.reference, im.reference)


def test_iter_axis():
    # axis iteration helper function.  This function also tests rollimg,
    # because iter_axis uses rollimg
    iter_axis = image.iter_axis
    data = np.arange(24).reshape((4,3,2))
    img = Image(data, AffineTransform('ijk', 'xyz', np.eye(4)))
    for ax_id, ax_no in (('i',0), ('j',1), ('k',2),
                        ('x',0), ('y',1), ('z',2),
                        (0,0), (1,1), (2,2),
                        (-1,2)):
        slices = list(iter_axis(img, ax_id))
        expected_shape = list(data.shape)
        g_len = expected_shape.pop(ax_no)
        assert_equal(len(slices), g_len)
        for s in slices:
            assert_equal(list(s.shape), expected_shape)
        # test asarray
        slicer = [slice(None) for i in range(data.ndim)]
        for i, s in enumerate(iter_axis(img, ax_id, asarray=True)):
            slicer[ax_no] = i
            assert_array_equal(s, data[slicer])


def test_rollaxis():
    data = np.random.standard_normal((3,4,7,5))
    im = Image(data, AffineTransform.from_params('ijkl', 'xyzt', np.diag([1,2,3,4,1])))
    # for the inverse we must specify an integer
    assert_raises(ValueError, image.rollaxis, im, 'i', True)
    # Check that rollaxis preserves diagonal affines, as claimed
    assert_almost_equal(image.rollaxis(im, 1).affine, np.diag([2,1,3,4,1]))
    assert_almost_equal(image.rollaxis(im, 2).affine, np.diag([3,1,2,4,1]))
    assert_almost_equal(image.rollaxis(im, 3).affine, np.diag([4,1,2,3,1]))
    # Check that ambiguous axes raise an exception
    # 'l' appears both as an axis and a reference coord name
    # and in different places
    im_amb = Image(data, AffineTransform.from_params('ijkl', 'xylt', np.diag([1,2,3,4,1])))
    assert_raises(ValueError, image.rollaxis, im_amb, 'l')
    # But if it's unambiguous, then
    # 'l' can appear both as an axis and a reference coord name
    im_unamb = Image(data, AffineTransform.from_params('ijkl', 'xyzl', np.diag([1,2,3,4,1])))
    im_rolled = image.rollaxis(im_unamb, 'l')
    assert_almost_equal(im_rolled.get_data(),
                        im_unamb.get_data().transpose([3,0,1,2]))
    for i, o, n in zip('ijkl', 'xyzt', range(4)):
        im_i = image.rollaxis(im, i)
        im_o = image.rollaxis(im, o)
        im_n = image.rollaxis(im, n)
        assert_almost_equal(im_i.get_data(), im_o.get_data())
        assert_almost_equal(im_i.affine, im_o.affine)
        assert_almost_equal(im_n.get_data(), im_o.get_data())
        for _im in [im_n, im_o, im_i]:
            im_n_inv = image.rollaxis(_im, n, inverse=True)
            assert_almost_equal(im_n_inv.affine, im.affine)
            assert_almost_equal(im_n_inv.get_data(), im.get_data())


def test_is_image():
    # Test is_image check
    arr = np.arange(24).reshape((2,3,4))
    cmap = AffineTransform(CoordinateSystem('ijk'),
                           CoordinateSystem('xyz'),
                           np.eye(4))
    img = Image(arr, cmap)
    assert_true(is_image(img))
    assert_false(is_image(object()))
    class C(object):
        def get_data(self): pass
    c = C()
    assert_false(is_image(c))
    c.coordmap = None
    assert_false(is_image(c))
    c.metadata = None
    assert_true(is_image(c))


def test_rollimg():
    AT = AffineTransform
    data = np.random.standard_normal((3,4,7,5))
    aff = np.diag([1,2,3,4,1])
    im = Image(data, AT('ijkl', 'xyzt', aff))
    # No attempt to preserve the diagonal
    im1 = rollimg(im, 1)
    assert_equal(im1.coordmap, rollimg(im, 'j').coordmap)
    assert_equal(im1.coordmap, rollimg(im, 'y').coordmap)
    assert_equal(im1.coordmap, rollimg(im, -3).coordmap)
    assert_equal(im1.coordmap,
                 AT('jikl', 'xyzt', aff[:, (1, 0, 2, 3, 4)]))
    assert_array_equal(im1.get_data(), np.rollaxis(data, 1))
    im2 = rollimg(im, 2)
    assert_equal(im2.coordmap, rollimg(im, 'k').coordmap)
    assert_equal(im2.coordmap, rollimg(im, 'z').coordmap)
    assert_equal(im2.coordmap, rollimg(im, -2).coordmap)
    assert_equal(im2.coordmap,
                 AT('kijl', 'xyzt', aff[:, (2, 0, 1, 3, 4)]))
    assert_array_equal(im2.get_data(), np.rollaxis(data, 2))
    im3 = rollimg(im, 3)
    assert_equal(im3.coordmap, rollimg(im, 'l').coordmap)
    assert_equal(im3.coordmap, rollimg(im, 't').coordmap)
    assert_equal(im3.coordmap, rollimg(im, -1).coordmap)
    assert_equal(im3.coordmap,
                        AT('lijk', 'xyzt', aff[:, (3, 0, 1, 2, 4)]))
    assert_array_equal(im3.get_data(), np.rollaxis(data, 3))
    # We can roll to before a specified axis
    im31 = rollimg(im, 3, 1)
    assert_equal(im31.coordmap, rollimg(im, 'l', 'j').coordmap)
    assert_equal(im31.coordmap, rollimg(im, 't', 'y').coordmap)
    assert_equal(im31.coordmap, rollimg(im, 't', 'j').coordmap)
    assert_equal(im31.coordmap, rollimg(im, 'l', 'y').coordmap)
    assert_equal(im31.coordmap, rollimg(im, -1, 'y').coordmap)
    assert_equal(im31.coordmap, rollimg(im, -1, -3).coordmap)
    assert_equal(im31.coordmap,
                 AT('iljk', 'xyzt', aff[:, (0, 3, 1, 2, 4)]))
    assert_array_equal(im31.get_data(), np.rollaxis(data, 3, 1))
    # Check that ambiguous axes raise an exception; 'l' appears both as an axis
    # and a reference coord name and in different places
    im_amb = Image(data, AT('ijkl', 'xylt', np.diag([1,2,3,4,1])))
    assert_raises(AxisError, rollimg, im_amb, 'l')
    # But if it's unambiguous, then 'l' can appear both as an axis and a
    # reference coord name
    im_unamb = Image(data, AT('ijkl', 'xyzl', np.diag([1,2,3,4,1])))
    im_rolled = rollimg(im_unamb, 'l')
    assert_array_equal(im_rolled.get_data(),
                       im_unamb.get_data().transpose([3,0,1,2]))
    # Zero row / col means we can't find an axis mapping, when fix0 is false
    aff_z = np.diag([1, 2, 3, 0, 1])
    im_z = Image(data, AT('ijkl', 'xyzt', aff_z))
    assert_raises(AxisError, rollimg, im_z, 't', fix0=False)
    # But we can work it out if we turn on our zero detector
    assert_equal(rollimg(im_z, 't', fix0=True).coordmap,
                 AT('lijk', 'xyzt', aff_z[:, (3, 0, 1, 2, 4)]))
    # That's the default
    assert_equal(rollimg(im_z, 't').coordmap,
                 AT('lijk', 'xyzt', aff_z[:, (3, 0, 1, 2, 4)]))
    # Non square is OK
    aff_r = np.array([[1, 0, 0, 10],
                      [0, 2, 0, 11],
                      [0, 0, 2, 12],
                      [0, 0, 0, 13],
                      [0, 0, 0, 1]])
    im_r = Image(data[:,:,:,0], AT('ijk', 'xyzt', aff_r))
    assert_equal(rollimg(im_r, 'k').coordmap,
                 AT('kij', 'xyzt', aff_r[:, (2, 0, 1, 3)]))
    # Unless you're tring to get at the dropped input dimension of course
    assert_raises(AxisError, rollimg, im_r, 't')
    # Another check for integers, input names, output names, reversing
    for i, o, n in zip('ijkl', 'xyzt', range(4)):
        im_i = rollimg(im, i)
        im_o = rollimg(im, o)
        im_n = rollimg(im, n)
        assert_array_equal(im_i.get_data(), im_o.get_data())
        assert_array_equal(im_i.affine, im_o.affine)
        assert_array_equal(im_n.get_data(), im_o.get_data())
        for _im in [im_n, im_o, im_i]:
            # We're rollimg back.  We want to roll the new axis 0 back to where
            # it started, which was position n
            im_n_inv = rollimg(_im, 0, n + 1)
            assert_array_equal(im_n_inv.affine, im.affine)
            assert_array_equal(im_n_inv.get_data(), im.get_data())


def test_rollimg_rollaxis():
    # Check that rollimg and rollaxis do the same
    AT = AffineTransform
    data = np.random.standard_normal((3,4,7,5))
    aff = np.diag([1,2,3,4,1])
    img = Image(data, AT('ijkl', 'xyzt', aff))
    for axis in range(4) + range(-3, -1):
        rdata = np.rollaxis(data, axis)
        rimg = rollimg(img, axis)
        assert_array_equal(rdata, rimg.get_data())
        for start in range(4) + range(-3, -1):
            rdata = np.rollaxis(data, axis, start)
            rimg = rollimg(img, axis, start)
            assert_array_equal(rdata, rimg.get_data())


def test_rollaxis_inverse():
    # Test deprecated image rollaxis with inverse
    AT = AffineTransform
    data = np.random.standard_normal((3,4,7,5))
    aff = np.diag([1,2,3,4,1])
    img = Image(data, AT('ijkl', 'xyzt', aff))
    for axis in range(4) + range(-3, -1):
        rimg = image.rollaxis(img, axis)
        rdata = np.rollaxis(data, axis)
        assert_array_equal(rdata, rimg.get_data())
        rrimg = image.rollaxis(rimg, axis, inverse=True)
        assert_array_equal(data, rrimg.get_data())



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np
import scipy.stats

class Link(object):

    """
    A generic link function for one-parameter exponential
    family, with call, inverse and deriv methods.

    """

    def initialize(self, Y):
        return np.asarray(Y).mean() * np.ones(Y.shape)

    def __call__(self, p):
        return NotImplementedError

    def inverse(self, z):
        return NotImplementedError

    def deriv(self, p):
        return NotImplementedError


class Logit(Link):

    """
    The logit transform as a link function:

    g'(x) = 1 / (x * (1 - x))
    g^(-1)(x) = exp(x)/(1 + exp(x))

    """

    tol = 1.0e-10

    def clean(self, p):
        """
        Clip logistic values to range (tol, 1-tol)

        INPUTS:
           p     -- probabilities

        OUTPUTS: pclip
           pclip -- clipped probabilities
        """

        return np.clip(p, Logit.tol, 1. - Logit.tol)

    def __call__(self, p):
        """
        Logit transform

        g(p) = log(p / (1 - p))

        INPUTS:
           p   -- probabilities

        OUTPUTS: z
           z   -- logit transform of p

        """

        p = self.clean(p)
        return np.log(p / (1. - p))

    def inverse(self, z):
        """
        Inverse logit transform

        h(z) = exp(z)/(1+exp(z))

        INPUTS:
           z -- logit transform of p

        OUTPUTS: p
           p   -- probabilities

        """
        t = np.exp(z)
        return t / (1. + t)

    def deriv(self, p):

        """
        Derivative of logit transform

        g(p) = 1 / (p * (1 - p))

        INPUTS:
           p   -- probabilities

        OUTPUTS: y
           y   -- derivative of logit transform of p

        """
        p = self.clean(p)
        return 1. / (p * (1 - p))

logit = Logit()

class Power(Link):

    """
    The power transform as a link function:

    g(x) = x**power

    """

    def __init__(self, power=1.):
        self.power = power

    def __call__(self, x):
        """
        Power transform

        g(x) = x**self.power

        INPUTS:
           x   -- mean parameters

        OUTPUTS: z
           z   -- power transform of x

        """

        return np.power(x, self.power)

    def inverse(self, z):
        """
        Inverse of power transform

        g(x) = x**(1/self.power)

        INPUTS:
           z   -- linear predictors in GLM

        OUTPUTS: x
           x   -- mean parameters

        """
        return np.power(z, 1. / self.power)

    def deriv(self, x):
        """
        Derivative of power transform

        g(x) = self.power * x**(self.power - 1)

        INPUTS:
           x   -- mean parameters

        OUTPUTS: z
           z   -- derivative of power transform of x

        """

        return self.power * np.power(x, self.power - 1)

inverse = Power(power=-1.)
inverse.__doc__ = """

The inverse transform as a link function:

g(x) = 1 / x
"""

sqrt = Power(power=0.5)
sqrt.__doc__ = """

The square-root transform as a link function:

g(x) = sqrt(x)
"""

inverse_squared = Power(power=-2.)
inverse_squared.__doc__ = """

The inverse squared transform as a link function:

g(x) = 1 / x**2
"""

identity = Power(power=1.)
identity.__doc__ = """

The identity transform as a link function:

g(x) = x
"""

class Log(Link):

    """
    The log transform as a link function:

    g(x) = log(x)

    """

    tol = 1.0e-10

    def clean(self, x):
        return np.clip(x, Logit.tol, np.inf)

    def __call__(self, x, **extra):
        """
        Log transform

        g(x) = log(x)

        INPUTS:
           x   -- mean parameters

        OUTPUTS: z
           z   -- log(x)

        """
        x = self.clean(x)
        return np.log(x)

    def inverse(self, z):
        """
        Inverse of log transform

        g(x) = exp(x)

        INPUTS:
           z   -- linear predictors in GLM

        OUTPUTS: x
           x   -- exp(z)

        """
        return np.exp(z)

    def deriv(self, x):
        """
        Derivative of log transform

        g(x) = 1/x

        INPUTS:
           x   -- mean parameters

        OUTPUTS: z
           z   -- derivative of log transform of x

        """

        x = self.clean(x)
        return 1. / x

log = Log()

class CDFLink(Logit):

    """
    The use the CDF of a scipy.stats distribution as a link function:

    g(x) = dbn.ppf(x)

    """

    def __init__(self, dbn=scipy.stats.norm):
        self.dbn = dbn

    def __call__(self, p):
        """
        CDF link

        g(p) = self.dbn.pdf(p)

        INPUTS:
           p   -- mean parameters

        OUTPUTS: z
           z   -- derivative of CDF transform of p

        """
        p = self.clean(p)
        return self.dbn.ppf(p)

    def inverse(self, z):
        """
        Derivative of CDF link

        g(z) = self.dbn.cdf(z)

        INPUTS:
           z   -- linear predictors in GLM

        OUTPUTS: p
           p   -- inverse of CDF link of z

        """
        return self.dbn.cdf(z)

    def deriv(self, p):
        """
        Derivative of CDF link

        g(p) = 1/self.dbn.pdf(self.dbn.ppf(p))

        INPUTS:
           x   -- mean parameters

        OUTPUTS: z
           z   -- derivative of CDF transform of x

        """
        p = self.clean(p)
        return 1. / self.dbn.pdf(self(p))

probit = CDFLink()
probit.__doc__ = """

The probit (standard normal CDF) transform as a link function:

g(x) = scipy.stats.norm.ppf(x)

"""

cauchy = CDFLink(dbn=scipy.stats.cauchy)
cauchy.__doc__ = """

The Cauchy (standard Cauchy CDF) transform as a link function:

g(x) = scipy.stats.cauchy.ppf(x)

"""

class CLogLog(Logit):

    """
    The complementary log-log transform as a link function:

    g(x) = log(-log(x))

    """

    def __call__(self, p):
        """
        C-Log-Log transform

        g(p) = log(-log(p))

        INPUTS:
           p   -- mean parameters

        OUTPUTS: z
           z   -- log(-log(p))

        """
        p = self.clean(p)
        return np.log(-np.log(p))

    def inverse(self, z):
        """
        Inverse of C-Log-Log transform

        g(z) = exp(-exp(z))

        INPUTS:
           z   -- linear predictor scale

        OUTPUTS: p
           p   -- mean parameters

        """
        return np.exp(-np.exp(z))

    def deriv(self, p):
        """
        Derivatve of C-Log-Log transform

        g(p) = - 1 / (log(p) * p)

        INPUTS:
           p   -- mean parameters

        OUTPUTS: z
           z   --  - 1 / (log(p) * p)

        """
        p = self.clean(p)
        return -1. / (np.log(p) * p)

cloglog = CLogLog()



""" Tests for regression module
"""

import numpy as np

from nipy.algorithms.statistics.api  import OLSModel
from ..outputters import output_T, output_F

from nose.tools import assert_true, assert_equal, assert_raises

from numpy.testing import (assert_array_almost_equal,
                           assert_array_equal)


N = 10
X = np.c_[np.linspace(-1,1,N), np.ones((N,))]
RNG = np.random.RandomState(20110901)
Y = RNG.normal(size=(10,1)) * 10 + 100
MODEL = OLSModel(X)
RESULTS = MODEL.fit(Y)
C1 = [1, 0]


def test_model():
    # Check basics about the model fit
    # Check we fit the mean
    assert_array_almost_equal(RESULTS.theta[1], np.mean(Y))


def test_output_T():
    # Check we get required outputs
    res = RESULTS.Tcontrast(C1) # all return values
    # default is all return values
    assert_array_almost_equal([res.effect, res.sd, res.t],
                              output_T(RESULTS, C1))
    assert_array_almost_equal([res.effect, res.sd, res.t],
                              output_T(RESULTS, C1, ('effect', 'sd', 't')))
    # Input order determines return order
    assert_array_almost_equal([res.t, res.effect, res.sd],
                              output_T(RESULTS, C1, ('t', 'effect', 'sd')))
    # And can select inputs
    assert_array_almost_equal([res.t],
                              output_T(RESULTS, C1, ('t',)))
    assert_array_almost_equal([res.sd],
                              output_T(RESULTS, C1, ('sd',)))
    assert_array_almost_equal([res.effect],
                              output_T(RESULTS, C1, ('effect',)))


def test_output_F():
    # Test output_F convenience function
    rng = np.random.RandomState(ord('F'))
    Y = rng.normal(size=(10,1)) * 10 + 100
    X = np.c_[rng.normal(size=(10,3)), np.ones((N,))]
    c1 = np.zeros((X.shape[1],))
    c1[0] = 1
    model = OLSModel(X)
    results = model.fit(Y)
    # Check we get required outputs
    exp_f = results.t(0) **2
    assert_array_almost_equal(exp_f, output_F(results, c1))



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import with_statement

import numpy as np

from nipy.io.api import load_image
from nipy.core.image.image import rollimg

from  .. import model
from ..model import ModelOutputImage, estimateAR
from ...api import FmriImageList

from nipy.algorithms.statistics.models.regression import (
    OLSModel, ar_bias_corrector, ar_bias_correct)
from nipy.algorithms.statistics.formula.formulae import(
    Formula, Term, make_recarray)


from nibabel.tmpdirs import InTemporaryDirectory

from nose.tools import assert_raises, assert_true, assert_equal

from numpy.testing import assert_array_equal, assert_array_almost_equal
from nipy.testing import funcfile, anatfile

def test_model_out_img():
    # Model output image
    cmap = load_image(anatfile).coordmap
    shape = (2,3,4)
    fname = 'myfile.nii'
    with InTemporaryDirectory():
        moi = ModelOutputImage(fname, cmap, shape)
        for i in range(shape[0]):
            moi[i] = i
        for i in range(shape[0]):
            assert_array_equal(moi[i], i)
        moi.save()
        assert_raises(ValueError, moi.__setitem__, 0, 1)
        assert_raises(ValueError, moi.__getitem__, 0)
        new_img = load_image(fname)
        for i in range(shape[0]):
            assert_array_equal(new_img[i].get_data(), i)
        del new_img


def test_run():
    ar1_fname = 'ar1_out.nii'
    funcim = load_image(funcfile)
    fmriims = FmriImageList.from_image(funcim, volume_start_times=2.)
    one_vol = fmriims[0]
    # Formula - with an intercept
    t = Term('t')
    f = Formula([t, t**2, t**3, 1])
    # Design matrix and contrasts
    time_vector = make_recarray(fmriims.volume_start_times, 't')
    con_defs = dict(c=t, c2=t+t**2)
    desmtx, cmatrices = f.design(time_vector, contrasts=con_defs)

    # Run with Image and ImageList
    for inp_img in (rollimg(funcim, 't'), fmriims):
        with InTemporaryDirectory():
            # Run OLS model
            outputs = []
            outputs.append(model.output_AR1(ar1_fname, fmriims))
            outputs.append(model.output_resid('resid_OLS_out.nii', fmriims))
            ols = model.OLS(fmriims, f, outputs)
            ols.execute()
            # Run AR1 model
            outputs = []
            outputs.append(
                model.output_T('T_out.nii', cmatrices['c'], fmriims))
            outputs.append(
                model.output_F('F_out.nii', cmatrices['c2'], fmriims))
            outputs.append(
                model.output_resid('resid_AR_out.nii', fmriims))
            rho = load_image(ar1_fname)
            ar = model.AR1(fmriims, f, rho, outputs)
            ar.execute()
            f_img = load_image('F_out.nii')
            assert_equal(f_img.shape, one_vol.shape)
            f_data = f_img.get_data()
            assert_true(np.all((f_data>=0) & (f_data<30)))
            resid_img = load_image('resid_AR_out.nii')
            assert_equal(resid_img.shape, funcim.shape)
            assert_array_almost_equal(np.mean(resid_img.get_data()), 0, 3)
            e_img = load_image('T_out_effect.nii')
            sd_img = load_image('T_out_sd.nii')
            t_img = load_image('T_out_t.nii')
            t_data = t_img.get_data()
            assert_array_almost_equal(t_data,
                                      e_img.get_data() / sd_img.get_data())
            assert_true(np.all(np.abs(t_data) < 6))
            # Need to delete to help windows delete temporary files
            del rho, resid_img, f_img, e_img, sd_img, t_img, f_data, t_data


def test_ar_modeling():
    # Compare against standard routines
    rng = np.random.RandomState(20110903)
    N = 10
    Y = rng.normal(size=(N,1)) * 10 + 100
    X = np.c_[np.linspace(-1,1,N), np.ones((N,))]
    my_model = OLSModel(X)
    results = my_model.fit(Y)
    # fmristat wrapper
    rhos = estimateAR(results.resid, my_model.design, order=2)
    assert_equal(rhos.shape, (2,))
    assert_true(np.all(np.abs(rhos <= 1)))
    # standard routine
    rhos2 = ar_bias_correct(results, 2)
    assert_array_almost_equal(rhos, rhos2, 8)
    # Make 2D and 3D Y
    Y = rng.normal(size=(N,4)) * 10 + 100
    results = my_model.fit(Y)
    rhos = estimateAR(results.resid, my_model.design, order=2)
    assert_equal(rhos.shape, (2,4))
    assert_true(np.all(np.abs(rhos <= 1)))
    rhos2 = ar_bias_correct(results, 2)
    assert_array_almost_equal(rhos, rhos2, 8)
    # 3D
    results.resid = np.reshape(results.resid, (N,2,2))
    rhos = estimateAR(results.resid, my_model.design, order=2)
    assert_equal(rhos.shape, (2,2,2))
    assert_true(np.all(np.abs(rhos <= 1)))
    rhos2 = ar_bias_correct(results, 2)
    assert_array_almost_equal(rhos, rhos2, 8)



"""
This module defines the StructuredDomain class,
that represents a generic neuroimaging kind of domain
This is meant to provide a unified API to deal with n-d imaged and meshes.


Author: Bertrand Thirion, 2010
"""
import numpy as np
import scipy.sparse as sp

from nibabel import load, Nifti1Image, save

from nipy.algorithms.graph import WeightedGraph, wgraph_from_coo_matrix, \
    wgraph_from_3d_grid

##############################################################
# Ancillary functions
##############################################################


def smatrix_from_3d_array(mask, nn=18):
    """Create a sparse adjacency matrix from an array

    Parameters
    ----------
    mask : 3d array,
           input array, interpreted as a mask
    nn: int, optional
        3d neighboring system to be chosen within {6, 18, 26}

    Returns
    -------
    coo_mat: a sparse coo matrix,
             adjacency of the neighboring system

    """
    ijk = np.array(np.where(mask)).T
    return smatrix_from_3d_idx(ijk, nn)


def smatrix_from_3d_idx(ijk, nn=18):
    """Create a sparse adjacency matrix from 3d index system

    Parameters
    ----------
    idx:array of shape (n_samples, 3), type int
        indexes of certain positions in a 3d space
    nn: int, optional
        3d neighboring system to be chosen within {6, 18, 26}

    Returns
    -------
    coo_mat: a sparse coo matrix,
             adjacency of the neighboring system

    """
    G = wgraph_from_3d_grid(ijk, nn)
    return G.to_coo_matrix()


def smatrix_from_nd_array(mask, nn=0):
    """Create a sparse adjacency matrix from an arbitrary nd array

    Parameters
    ----------
    mask : nd array,
           input array, interpreted as a mask
    nn: int, optional
        nd neighboring system, unsused at the moment

    Returns
    -------
    coo_mat: a sparse coo matrix,
             adjacency of the neighboring system

    """
    idx = np.array(np.where(mask)).T
    return smatrix_from_nd_idx(idx, nn)


def smatrix_from_nd_idx(idx, nn=0):
    """Create a sparse adjacency matrix from nd index system

    Parameters
    ----------
    idx:array of shape (n_samples, dim), type int
        indexes of certain positions in a nd space
    nn: int, optional
        nd neighboring system, unused at the moment

    Returns
    -------
    coo_mat: a sparse coo matrix,
             adjacency of the neighboring system

    """
    n = idx.shape[0]
    dim = idx.shape[1]
    nidx = idx - idx.min(0)
    eA = []
    eB = []

    # compute the edges in each possible direction
    for d in range(dim):
        mi = nidx.max(0) + 2
        a = np.hstack((1, np.cumprod(mi[:dim - 1])))
        v1 = np.dot(nidx, a)
        assert(np.size(v1) == np.size(np.unique(v1)))
        o1 = np.argsort(v1)
        sv1 = v1[o1]
        nz = np.squeeze(np.nonzero(sv1[:n - 1] - sv1[1:] == - 1))
        nz = np.reshape(nz, np.size(nz))
        eA.append(o1[nz])
        eB.append(o1[nz + 1])
        nidx = np.roll(nidx, 1, 1)

    eA = np.concatenate(eA)
    eB = np.concatenate(eB)
    E = 2 * np.size(eA)
    # create a graph structure
    if E == 0:
        return sp.coo_matrix((n, n))

    edges = np.vstack((np.hstack((eA, eB)), np.hstack((eB, eA)))).T
    weights = np.ones(E)
    G = WeightedGraph(n, edges, weights)
    return G.to_coo_matrix()


def array_affine_coord(mask, affine):
    """Compute coordinates from a boolean array and an affine transform

    Parameters
    ----------
    mask: nd array,
           input array, interpreted as a mask
    affine: (n+1, n+1) matrix,
            affine transform that maps the mask points to some embedding space

    Returns
    -------
    coords: array of shape(sum(mask>0), n),
            the computed coordinates

    """
    idx = np.array(np.where(mask)).T
    return idx_affine_coord(idx, affine)


def idx_affine_coord(idx, affine):
    """Compute coordinates from a set of indexes and an affine transform

    Parameters
    ----------
    idx:array of shape (n_samples, dim), type int
        indexes of certain positions in a nd space
    affine: (n+1, n+1) matrix,
            affine transform that maps the mask points to some embedding space

    Returns
    -------
    coords: array of shape(sum(mask>0), n),
            the computed coordinates

    """
    size = idx.shape[0]
    hidx = np.hstack((idx, np.ones((size, 1))))
    coord = np.dot(hidx, affine.T)[:, 0:-1]
    return coord


def reduce_coo_matrix(mat, mask):
    """Reduce a supposedly coo_matrix to the vertices in the mask

    Parameters
    ----------
    mat: sparse coo_matrix,
         input matrix
    mask: boolean array of shape mat.shape[0],
          desired elements

    """
    G = wgraph_from_coo_matrix(mat)
    K = G.subgraph(mask)
    if K is None:
        return None
    return K.to_coo_matrix()


#################################################################
# Functions to instantiate StructuredDomains
#################################################################
def domain_from_binary_array(mask, affine=None, nn=0):
    """Return a StructuredDomain from an n-d array

    Parameters
    ----------
    mask: np.array instance
          a supposedly boolean array that repesents the domain
    affine: np.array, optional
            affine transform that maps the array coordinates
            to some embedding space
            by default, this is np.eye(dim+1, dim+1)
    nn: neighboring system considered
        unsued at the moment

    """
    dim = len(mask.shape)
    if affine is None:
        affine = np.eye(dim + 1)
    mask = mask > 0
    vol = np.absolute(np.linalg.det(affine)) * np.ones(np.sum(mask))
    coord = array_affine_coord(mask, affine)
    topology = smatrix_from_nd_array(mask)
    return StructuredDomain(dim, coord, vol, topology)


def domain_from_image(mim, nn=18):
    """Return a StructuredDomain instance from the input mask image

    Parameters
    ----------
    mim: NiftiIImage instance, or string path toward such an image
         supposedly a mask (where is used to crate the DD)
    nn: int, optional
        neighboring system considered from the image
        can be 6, 18 or 26

    Returns
    -------
    The corresponding StructuredDomain instance

    """
    if isinstance(mim, basestring):
        iim = load(mim)
    else:
        iim = mim
    return domain_from_binary_array(iim.get_data(), iim.get_affine(), nn)


def grid_domain_from_binary_array(mask, affine=None, nn=0):
    """Return a NDGridDomain from an n-d array

    Parameters
    ----------
    mask: np.array instance
          a supposedly boolean array that repesents the domain
    affine: np.array, optional
            affine transform that maps the array coordinates
            to some embedding space
            by default, this is np.eye(dim+1, dim+1)
    nn: neighboring system considered
        unsued at the moment

    """
    dim = len(mask.shape)
    shape = mask.shape
    if affine is None:
        affine = np.eye(dim + 1)

    mask = mask > 0
    ijk = np.array(np.where(mask)).T
    vol = np.absolute(np.linalg.det(affine[:3, 0:3])) * np.ones(np.sum(mask))
    topology = smatrix_from_nd_idx(ijk, nn)
    return NDGridDomain(dim, ijk, shape, affine, vol, topology)


def grid_domain_from_image(mim, nn=18):
    """Return a NDGridDomain instance from the input mask image

    Parameters
    ----------
    mim: NiftiIImage instance, or string path toward such an image
         supposedly a mask (where is used to crate the DD)
    nn: int, optional
        neighboring system considered from the image
        can be 6, 18 or 26

    Returns
    -------
    The corresponding NDGridDomain instance

    """
    if isinstance(mim, basestring):
        iim = load(mim)
    else:
        iim = mim
    return grid_domain_from_binary_array(iim.get_data(), iim.get_affine(), nn)


def grid_domain_from_shape(shape, affine=None):
    """Return a NDGridDomain from an n-d array

    Parameters
    ----------
    shape: tuple
      the shape of a rectangular domain.
    affine: np.array, optional
      affine transform that maps the array coordinates
      to some embedding space.
      By default, this is np.eye(dim+1, dim+1)

    """
    dim = len(shape)
    if affine is None:
        affine = np.eye(dim + 1)

    rect = np.ones(shape)
    ijk = np.array(np.where(rect)).T
    vol = np.absolute(np.linalg.det(affine[:3, 0:3])) * np.ones(np.sum(rect))
    topology = smatrix_from_nd_idx(ijk, 0)
    return NDGridDomain(dim, ijk, shape, affine, vol, topology)


################################################################
# Domain from mesh
################################################################


class MeshDomain(object):
    """
    temporary class to handle meshes
    """

    def __init__(self, coord, triangles):
        """Initialize mesh domain instance

        Parameters
        ----------
        coord: array of shape (n_vertices, 3),
               the node coordinates
        triangles: array of shape(n_triables, 3),
                   indices of the nodes per triangle

        """
        self.coord = coord
        self.triangles = triangles
        self.V = len(coord)
        # fixme: implement consistency checks

    def area(self):
        """Return array of areas for each node

        Returns
        -------
        area: array of shape self.V,
              area of each node

        """
        E = len(self.triangles)
        narea = np.zeros(self.V)

        def _area(a, b):
            """Area spanned by the vectors(a,b) in 3D
            """
            c = np.array([a[1] * b[2] - a[2] * b[1],
                          - a[0] * b[2] + a[2] * b[0],
                          a[0] * b[1] - a[1] * b[0]])
            return np.sqrt((c ** 2).sum())

        for e in range(E):
            i, j, k = self.triangles[e]
            a = self.coord[i] - self.coord[k]
            b = self.coord[j] - self.coord[k]
            ar = _area(a, b)
            narea[i] += ar
            narea[j] += ar
            narea[k] += ar

        narea /= 6
        # because division by 2 has been 'forgotten' in area computation
        # the area of a triangle is divided into the 3 vertices
        return narea

    def topology(self):
        """Returns a sparse matrix that represents the connectivity in self
        """
        E = len(self.triangles)
        edges = np.zeros((3 * E, 2))
        weights = np.ones(3 * E)

        for i in range(E):
            sa, sb, sc = self.triangles[i]
            edges[3 * i] = np.array([sa, sb])
            edges[3 * i + 1] = np.array([sa, sc])
            edges[3 * i + 2] = np.array([sb, sc])

        G = WeightedGraph(self.V, edges, weights)

        # symmeterize the graph
        G = G.symmeterize()

        # remove redundant edges
        G = G.cut_redundancies()

        # make it a metric graph
        G.set_euclidian(self.coord)
        return G.to_coo_matrix()


def domain_from_mesh(mesh):
    """Instantiate a StructuredDomain from a gifti mesh

    Parameters
    ----------
    mesh: nibabel gifti mesh instance, or path to such a mesh

    """
    if isinstance(mesh, basestring):
        from nibabel.gifti import read
        mesh_ = read(mesh)
    else:
        mesh_ = mesh

    if len(mesh_.darrays) == 2:
        cor, tri = mesh_.darrays
    elif len(mesh_.darrays) == 3:
        cor, nor, tri = mesh_.darrays
    else:
        raise Exception("%d arrays in gifti file (case not handled)" \
                            % len(mesh_.darrays))
    mesh_dom = MeshDomain(cor.data, tri.data)

    vol = mesh_dom.area()
    topology = mesh_dom.topology()
    dim = 2
    return StructuredDomain(dim, mesh_dom.coord, vol, topology)


################################################################
# StructuredDomain class
################################################################


class DiscreteDomain(object):
    """
    Descriptor of a certain domain that consists of discrete elements that
    are characterized by a coordinate system and a topology:
    the coordinate system is specified through a coordinate array
    the topology encodes the neighboring system

    """

    def __init__(self, dim, coord, local_volume, id='', referential=''):
        """Initialize discrete domain instance

        Parameters
        ----------
        dim: int,
             the (physical) dimension of the domain
        coord: array of shape(size, em_dim),
               explicit coordinates of the domain sites
        local_volume: array of shape(size),
                      yields the volume associated with each site
        id: string, optional,
             domain identifier
        referential: string, optional,
                     identifier of the referential of the coordinates system

        Caveat
        ------
        em_dim may be greater than dim e.g. (meshes coordinate in 3D)

        """
        # dimension
        self.dim = dim

        # number of discrete elements
        self.size = coord.shape[0]

        # coordinate system
        if np.size(coord) == coord.shape[0]:
            coord = np.reshape(coord, (np.size(coord), 1))

        if np.size(coord) == 0:
            self.em_dim = dim
        else:
            self.em_dim = coord.shape[1]

        if self.em_dim < dim:
            raise ValueError('Embedding dimension cannot be smaller than dim')
        self.coord = coord

        # volume
        if np.size(local_volume) != self.size:
            raise ValueError("Inconsistent Volume size")
        if (local_volume < 0).any():
            raise ValueError('Volume should be positive')

        self.local_volume = np.ravel(local_volume)
        self.id = id
        self.referential = referential
        self.features = {}

    def copy(self):
        """Returns a copy of self
        """
        new_dom = DiscreteDomain(self.dim, self.coord.copy(),
                                  self.local_volume.copy(), self.id,
                                  self.referential)
        for fid in self.features.keys():
            new_dom.set_feature(fid, self.get_feature(fid).copy())
        return new_dom

    def get_coord(self):
        """Returns self.coord
        """
        return self.coord

    def get_volume(self):
        """Returns self.local_volume
        """
        return self.local_volume

    def connected_components(self):
        """returns a labelling of the domain into connected components
        """
        if self.topology is not None:
            return wgraph_from_coo_matrix(self.topology).cc()
        else:
            return []

    def mask(self, bmask, id=''):
        """Returns an DiscreteDomain instance that has been further masked
        """
        if bmask.size != self.size:
            raise ValueError('Invalid mask size')

        svol = self.local_volume[bmask]
        scoord = self.coord[bmask]
        DD = DiscreteDomain(self.dim, scoord, svol, id, self.referential)

        for fid in self.features.keys():
            f = self.features.pop(fid)
            DD.set_feature(fid, f[bmask])
        return DD

    def set_feature(self, fid, data, override=True):
        """Append a feature 'fid'

        Parameters
        ----------
        fid: string,
             feature identifier
        data: array of shape(self.size, p) or self.size
              the feature data

        """
        if data.shape[0] != self.size:
            raise ValueError('Wrong data size')

        if (fid in self.features) & (override == False):
            return

        self.features.update({fid: data})

    def get_feature(self, fid):
        """Return self.features[fid]
        """
        return self.features[fid]

    def representative_feature(self, fid, method):
        """Compute a statistical representative of the within-Foain feature

        Parameters
        ----------
        fid: string, feature id
        method: string, method used to compute a representative
                to be chosen among 'mean', 'max', 'median', 'min'

        """
        f = self.get_feature(fid)
        if method == "mean":
            return np.mean(f, 0)
        if method == "min":
            return np.min(f, 0)
        if method == "max":
            return np.max(f, 0)
        if method == "median":
            return np.median(f, 0)

    def integrate(self, fid):
        """Integrate  certain feature over the domain and returns the result

        Parameters
        ----------
        fid : string,  feature identifier,
              by default, the 1 function is integrataed, yielding domain volume

        Returns
        -------
        lsum = array of shape (self.feature[fid].shape[1]),
               the result

        """
        if fid == None:
            return np.sum(self.local_volume)
        ffid = self.features[fid]
        if np.size(ffid) == ffid.shape[0]:
            ffid = np.reshape(ffid, (self.size, 1))
        slv = np.reshape(self.local_volume, (self.size, 1))
        return np.sum(ffid * slv, 0)


class StructuredDomain(DiscreteDomain):
    """
    Besides DiscreteDomain attributed, StructuredDomain has a topology,
    which allows many operations (morphology etc.)
    """

    def __init__(self, dim, coord, local_volume, topology, did='',
                 referential=''):
        """Initialize structured domain instance

        Parameters
        ----------
        dim: int,
             the (physical) dimension of the domain
        coord: array of shape(size, em_dim),
               explicit coordinates of the domain sites
        local_volume: array of shape(size),
                      yields the volume associated with each site
        topology: sparse binary coo_matrix of shape (size, size),
                  that yields the neighboring locations in the domain
        did: string, optional,
             domain identifier
        referential: string, optional,
                     identifier of the referential of the coordinates system

        """
        DiscreteDomain.__init__(self, dim, coord, local_volume, id,
                                referential)

        # topology
        if topology is not None:
            if topology.shape != (self.size, self.size):
                raise ValueError('Incompatible shape for topological model')
        self.topology = topology

    def mask(self, bmask, did=''):
        """Returns a StructuredDomain instance that has been further masked
        """
        td = DiscreteDomain.mask(self, bmask)
        stopo = reduce_coo_matrix(self.topology, bmask)
        dd = StructuredDomain(self.dim, td.coord, td.local_volume,
                            stopo, did, self.referential)

        for fid in td.features.keys():
            dd.set_feature(fid, td.features.pop(fid))
        return dd


class NDGridDomain(StructuredDomain):
    """
    Particular instance of StructuredDomain, that receives
    3 additional variables:
    affine: array of shape (dim+1, dim+1),
            affine transform that maps points to a coordinate system
    shape: dim-tuple,
           shape of the domain
    ijk: array of shape(size, dim), int
         grid coordinates of the points

    This is to allow easy conversion to images when dim==3,
    and for compatibility with previous classes

    """

    def __init__(self, dim, ijk, shape, affine, local_volume, topology,
                referential=''):
        """Initialize ndgrid domain instance

        Parameters
        ----------
        dim: int,
             the (physical) dimension of the domain
        ijk: array of shape(size, dim), int
             grid coordinates of the points
        shape: dim-tuple,
           shape of the domain
        affine: array of shape (dim+1, dim+1),
            affine transform that maps points to a coordinate system
        local_volume: array of shape(size),
                      yields the volume associated with each site
        topology: sparse binary coo_matrix of shape (size, size),
                  that yields the neighboring locations in the domain
        referential: string, optional,
                     identifier of the referential of the coordinates system

        Fixme
        -----
        local_volume might be computed on-the-fly as |det(affine)|

        """
        # shape
        if len(shape) != dim:
            raise ValueError('Incompatible shape and dim')
        self.shape = shape

        # affine
        if affine.shape != (dim + 1, dim + 1):
            raise ValueError('Incompatible dim and affine')
        self.affine = affine

        # ijk
        if (np.size(ijk) == ijk.shape[0]) & (np.size(ijk) > 0):
            ijk = np.reshape(ijk, (ijk.size, 1))
            if (ijk.max(0) + 1 > shape).any():
                raise ValueError('Provided indices do not fit within shape')
        self.ijk = ijk

        # coord
        coord = idx_affine_coord(ijk, affine)

        StructuredDomain.__init__(self, dim, coord, local_volume, topology)

    def mask(self, bmask):
        """Returns an instance of self that has been further masked
        """
        if bmask.size != self.size:
            raise ValueError('Invalid mask size')

        svol = self.local_volume[bmask]
        stopo = reduce_coo_matrix(self.topology, bmask)
        sijk = self.ijk[bmask]
        DD = NDGridDomain(self.dim, sijk, self.shape, self.affine, svol,
                          stopo, self.referential)

        for fid in self.features.keys():
            f = self.features.pop(fid)
            DD.set_feature(fid, f[bmask])
        return DD

    def to_image(self, path=None, data=None):
        """Write itself as a binary image, and returns it

        Parameters
        ----------
        path: string, path of the output image, if any
        data: array of shape self.size,
              data to put in the nonzer-region of the image

        """
        if data is None:
            wdata = np.zeros(self.shape, np.int8)
        else:
            wdata = np.zeros(self.shape, data.dtype)
        wdata[self.ijk[:, 0], self.ijk[:, 1], self.ijk[:, 2]] = 1
        if data is not None:
            if data.size != self.size:
                raise ValueError('incorrect data size')
            wdata[wdata > 0] = data

        nim = Nifti1Image(wdata, self.affine)
        nim.get_header()['descrip'] = 'mask image representing domain %s' \
            % self.id
        if path is not None:
            save(nim, path)
        return nim

    def make_feature_from_image(self, path, fid=''):
        """Extract the information from an image to make it a domain a feature

        Parameters
        ----------
        path: string or Nifti1Image instance,
              the image from which one wished to extract data
        fid: string, optional
             identifier of the resulting feature.
             if '', the feature is not stored

        Returns
        -------
        the correponding set of values

        """
        if isinstance(path, basestring):
            nim = load(path)
        else:
            nim = path

        if (nim.get_affine() != self.affine).any():
            raise ValueError('nim and self do not have the same referential')

        data = nim.get_data()
        feature = data[self.ijk[:, 0], self.ijk[:, 1], self.ijk[:, 2]]
        if fid is not '':
            self.features[fid] = feature

        return feature



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The volume grid class. 

This class represents data lying on a (non rigid, non regular) grid embedded 
in a 3D world represented as a 3+D array.
"""

import copy as copy

import numpy as np
from scipy import ndimage

# Local imports
from .volume_data import VolumeData
from ..transforms.affine_utils import apply_affine, from_matrix_vector


################################################################################
# class `VolumeGrid`
################################################################################

class VolumeGrid(VolumeData):
    """ A class representing data stored in a 3+D array embedded in a 3D
        world.

        This object has data stored in an array-like multidimensional 
        indexable objects, with the 3 first dimensions corresponding to
        spatial axis and defining a 3D grid that may be non-regular or
        non-rigid.

        The object knows how the data is mapped to a 3D "real-world
        space", and how it can change real-world coordinate system. The
        transform mapping it to world is arbitrary, and thus the grid
        can be warped: in the world space, the grid may not be regular or
        orthogonal.

        Attributes
        -----------

        world_space: string 
            World space the data is embedded in. For instance `mni152`.

        metadata: dictionnary
            Optional, user-defined, dictionnary used to carry around
            extra information about the data as it goes through
            transformations. The consistency of this information is not
            maintained as the data is modified.

        _data: 
            Private pointer to the data.

        Notes
        ------

        The data is stored in an undefined way: prescalings might need to
        be applied to it before using it, or the data might be loaded on
        demand. The best practice to access the data is not to access the
        _data attribute, but to use the `get_data` method.

        If the transform associated with the image has no inverse
        mapping, data corresponding to a given world space position cannot
        be calulated. If it has no forward mapping, it is impossible to
        resample another dataset on the same support.
    """
    #---------------------------------------------------------------------------
    # Public methods -- VolumeGrid interface
    #---------------------------------------------------------------------------

    def __init__(self, data, transform, metadata=None,
                 interpolation='continuous'):
        """ The base image containing data.

            Parameters
            ----------

            data: ndarray
                n dimensional array giving the embedded data, with the 3
                first dimensions being spatial.
            transform: nipy transform object
                The transformation from voxel to world.
            metadata : dictionnary, optional
                Dictionnary of user-specified information to store with
                the image.
            interpolation : 'continuous' or 'nearest', optional
                Interpolation type used when calculating values in
                different word spaces.
        """
        if not interpolation in ('continuous', 'nearest'):
            raise ValueError('interpolation must be either continuous '
                             'or nearest')
        self._data       = data
        self._transform  = transform
        self.world_space = transform.output_space
        if metadata is None:
            metadata = dict()
        self.metadata = metadata
        self.interpolation = interpolation


    def as_volume_img(self, affine=None, shape=None, 
                                        interpolation=None, copy=True):
        if affine is None:
            affine = np.eye(3)
        if affine.shape[0] == 3 or shape is None:
            affine3d = affine[:3, :3]
            affine4d = np.eye(4)
            affine4d[:3, :3] = affine3d
            x, y, z = self.get_world_coords()
            x, y, z = apply_affine(x, y, z, np.linalg.inv(affine4d))
            xmin = x.min()
            ymin = y.min()
            zmin = z.min()
            if affine.shape[0] == 3:
                offset = np.array((xmin, ymin, zmin))
                offset = np.dot(affine3d, offset)
                affine = from_matrix_vector(affine3d, offset[:3])
            if shape is None:
                xmax = x.max()
                ymax = y.max()
                zmax = z.max()
                shape = (np.ceil(xmax - xmin)+1,
                         np.ceil(ymax - ymin)+1,
                         np.ceil(zmax - zmin)+1, )
        shape = list(shape)
        if not len(shape) == 3:
            raise ValueError('The shape specified should be the shape '
                'the 3D grid, and thus of length 3. %s was specified'
                % shape )
        x, y, z = np.indices(shape)
        x, y, z = apply_affine(x, y, z, affine)
        values = self.values_in_world(x, y, z)
        # We import late to avoid circular import
        from .volume_img import VolumeImg
        return VolumeImg(values, affine, 
                           self.world_space, metadata=self.metadata,
                           interpolation=self.interpolation)


    # Inherit docstring
    as_volume_img.__doc__ = VolumeData.as_volume_img.__doc__


    def get_world_coords(self):
        """ Return the data points coordinates in the world
            space.

            Returns
            --------
            x: ndarray
                x coordinates of the data points in world space
            y: ndarray
                y coordinates of the data points in world space
            z: ndarray
                z coordinates of the data points in world space

        """
        x, y, z = np.indices(self._data.shape[:3])
        return self.get_transform().mapping(x, y, z)


    # XXX: The docstring should be inherited


    def like_from_data(self, data):
        return self.__class__(data          = data,
                              transform     = copy.copy(self._transform),
                              metadata      = copy.copy(self.metadata),
                              interpolation = self.interpolation,
                              )

    # Inherit docstring
    like_from_data.__doc__ = VolumeData.like_from_data.__doc__


    def get_transform(self):
        """ Returns the transform object associated with the image which is a 
            general description of the mapping from the voxel space to the 
            world space.
            
            Returns
            -------
            transform : nipy.core.Transform object
        """
        return self._transform


    # Inherit docstring
    get_transform.__doc__ = VolumeData.get_transform.__doc__


    def values_in_world(self, x, y, z, interpolation=None):
        """ Return the values of the data at the world-space positions given by 
            x, y, z

            Parameters
            ----------
            x : number or ndarray
                x positions in world space, in other words milimeters
            y : number or ndarray
                y positions in world space, in other words milimeters.
                The shape of y should match the shape of x
            z : number or ndarray
                z positions in world space, in other words milimeters.
                The shape of z should match the shape of x
            interpolation : None, 'continuous' or 'nearest', optional
                Interpolation type used when calculating values in
                different word spaces. If None, the object's interpolation
                logic is used.

            Returns
            -------
            values : number or ndarray
                Data values interpolated at the given world position.
                This is a number or an ndarray, depending on the shape of
                the input coordinate.
        """
        interpolation_order = self._get_interpolation_order(interpolation)
        transform = self.get_transform()
        if transform.inverse_mapping is None:
            raise ValueError(
                "Cannot calculate the world values for volume data: mapping to "
                "word is not invertible."
                )
        x = np.atleast_1d(x)
        y = np.atleast_1d(y)
        z = np.atleast_1d(z)
        shape = list(x.shape)
        if not ((x.shape == y.shape) and (x.shape == z.shape)):
            raise ValueError('x, y and z shapes should be equal')
        x = x.ravel()
        y = y.ravel()
        z = z.ravel()
        i, j, k = transform.inverse_mapping(x, y, z)
        coords = np.c_[i, j, k].T
        # work round an ndimage deficiency in scipy <= 0.9.0.
        # See: https://github.com/scipy/scipy/pull/64
        if coords.dtype == np.dtype(np.intp):
            coords = coords.astype(np.dtype(coords.dtype.str))
        data = self.get_data()
        data_shape = list(data.shape)
        n_dims = len(data_shape)
        if n_dims > 3:
            # Iter in a set of 3D volumes, as the interpolation problem is 
            # separable in the extra dimensions. This reduces the
            # computational cost
            data = np.reshape(data, data_shape[:3] + [-1])
            data = np.rollaxis(data, 3)
            values = [ ndimage.map_coordinates(slice, coords,
                                               order=interpolation_order)
                       for slice in data]
            values = np.array(values)
            values = np.swapaxes(values, 0, -1)
            values = np.reshape(values, shape + data_shape[3:])
        else:
            values = ndimage.map_coordinates(data, coords,
                                        order=interpolation_order)
            values = np.reshape(values, shape)
        return values

    # Inherit docstring
    values_in_world.__doc__ = VolumeData.values_in_world.__doc__

